dnl APL_CONFIG_NICE(filename)
dnl Puts the configure options into a convenient shell script
AC_DEFUN(APL_CONFIG_NICE,[
if test -z "$no_recursion"; then
  rm -f $1
  cat >$1<<EOF
#! /bin/sh
#
# Created by configure

EOF

  for var in CFLAGS CXXFLAGS CPPFLAGS LDFLAGS LIBS; do
    eval val="\$$var"
    if test -n "$val"; then
      eval echo "$var=\'$val\' \\\\" >> $1
    fi
  done
    
  for arg in [$]0 "[$]@"; do
    echo "'[$]arg' \\" >> $1
  done
  echo '"[$]@"' >> $1
  chmod +x $1
fi
])

dnl ARG_HEADERS
dnl Disables installing headers conditionally
AC_DEFUN(ARG_HEADERS,[
AC_ARG_ENABLE(headers,
[  --disable-headers       Don't install headers],
[
  install_headers=$enableval
],[
  install_headers=yes
])
if test "$install_headers" = "yes"; then
  INSTALL_H='$(EXTRA_HEADERS)'
else
  INSTALL_H=''
fi
AC_SUBST(INSTALL_H)
])

dnl ARG_SQLXX
dnl Handles the --with-sqlxx option
AC_DEFUN(ARG_SQLXX,[
AC_ARG_WITH(sqlxx,
[  --with-sqlxx=DIR],
[
  CPPFLAGS="$CPPFLAGS -I$withval/include"
  LDFLAGS="$LDFLAGS -L$withval/lib"
  AC_MSG_CHECKING(for sqlxx prefix)
  AC_MSG_RESULT($withval)
])
AC_CHECK_HEADERS(sqlxx.h,[ ],[
  AC_MSG_ERROR([Sorry[,] I was not able to find the sqlxx header file. Please specify the sqlxx install prefix using --with-sqlxx=DIR])
])
])

dnl ARG_STRUTILSXX
dnl Handles the --with-strutilsxx option
AC_DEFUN(ARG_STRUTILSXX,[
AC_ARG_WITH(strutilsxx,
[  --with-strutilsxx=DIR],
[
  CPPFLAGS="$CPPFLAGS -I$withval/include"
  LDFLAGS="$LDFLAGS -L$withval/lib"
  AC_MSG_CHECKING(for strutilsxx prefix)
  AC_MSG_RESULT($withval)
])
AC_CHECK_HEADERS(strutilsxx.h,[ ],[
  AC_MSG_ERROR([Sorry[,] I was not able to find the strutilsxx header file. Please specify the strutilsxx install prefix using --with-strutilsxx=DIR])
])
])

dnl ARG_PHP_CONFIG
dnl Provides the definition of --with-php-config
AC_DEFUN(ARG_PHP_CONFIG,[
AC_ARG_WITH(php-config,
[  --with-php-config=PATH],
[
  PHP_CONFIG=$withval
])
])

dnl ARG_TEMPLATE_DIR
dnl Allows a user to configure the default template directory
AC_DEFUN(ARG_TEMPLATE_DIR,[
AC_ARG_WITH(template-dir,
[  --with-template-dir=PATH],
[
  APL_TEMPLATE_DIR=$withval
],[
  APL_TEMPLATE_DIR=$prefix/share/apl-themes
])
AC_SUBST(APL_TEMPLATE_DIR)
AC_DEFINE_UNQUOTED(APL_TEMPLATE_DIR, "$APL_TEMPLATE_DIR", [Where the templates are to be located])
AC_MSG_CHECKING(for the template directory)
AC_MSG_RESULT($APL_TEMPLATE_DIR)
])

dnl ARG_TEMPLATE_ALIAS
dnl Enables a user to configure the default theme alias
AC_DEFUN(ARG_TEMPLATE_ALIAS,[
AC_ARG_WITH(template-alias,
[  --with-template-alias=STRING],
[
  APL_TEMPLATE_ALIAS=$withval
],[
  APL_TEMPLATE_ALIAS="/apl-themes"
])
AC_DEFINE_UNQUOTED(APL_TEMPLATE_ALIAS, "$APL_TEMPLATE_ALIAS", [The absolute path to access the templates directory through the web-server])
AC_MSG_CHECKING(for web alias to the templates directory)
AC_MSG_RESULT($APL_TEMPLATE_ALIAS)
])

dnl CHECK_PHP
dnl Set up the environment to compile as a PHP extension
AC_DEFUN(CHECK_PHP,[

if test -z "$PHP_CONFIG"; then
  PHP_CONFIG=php-config
fi

prefix=`$PHP_CONFIG --prefix`

if test -z "$prefix"; then
  AC_MSG_ERROR(Could not run php-config. Please make sure that you installed PHP and that php-config is in your path or use the --with-php-config option)
fi

php_includes=`$PHP_CONFIG --includes`
php_extension_dir=`$PHP_CONFIG --extension-dir`
libdir="$php_extension_dir"

AC_MSG_CHECKING(for PHP prefix)
AC_MSG_RESULT($prefix)
AC_MSG_CHECKING(for PHP includes)
AC_MSG_RESULT($php_includes)
AC_MSG_CHECKING(for PHP extension directory)
AC_MSG_RESULT($php_extension_dir)

CPPFLAGS="$CPPFLAGS $php_includes"

AC_CHECK_HEADERS(Zend/modules.h Zend/zend_modules.h)

])

dnl AC_CHECK_DEBUG
dnl Set debug environment variable
AC_DEFUN(AC_CHECK_DEBUG,[
  AC_ARG_ENABLE(
    debug,
    [  --enable-debug          Enable debugging messages],
    [ 
      if test "x$enable_debug" = "xyes"
      then
        AC_DEFINE_UNQUOTED(APL_DEBUG, "debug", [Debug enabled])
      fi  
    ]
)])
