<?php
/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: dataform_simple.php,v 1.3 2002/01/15 19:40:50 ach Exp $ */

include("apl_config.php");

class CDatabase extends CAPLDatabase
{
  function CDatabase()
  {
    CAPLDatabase::CAPLDatabase();
    $this->setHostname(DB_HOSTNAME);
    $this->setUsername(DB_USERNAME);
    $this->setPassword(DB_PASSWORD);
    $this->setDriver(DB_DRIVER);
    $this->setDatabase(DB_DATABASE); 
    $this->setType(DB_TYPE);
  }
}

class CProductControl extends CAPLDBControl
{  
  function CProductControl($Database)
  {
    CAPLDBControl::CAPLDBControl($Database);
    $this->setTable("tblapldataform2product");
    $this->setPrimaryKey("productid");
  }
}

$DB = new CDatabase();
$Page = new CAPLPage();
$Page->Action = new CAPLActionHandler();

// Global Functions

function convert2Price($sKey, $sField, $sValue)
{
  if (!$sValue) $sValue = 0;
  return number_format($sValue, 2, ",",".");
}

function convert2Numeric($sKey, $sField, $sValue)
{
  $fValue = str_replace(",",".",str_replace(".","", $sValue));
  if ($fValue == "" || $fValue== ".") $fValue = 0;
  return $fValue;
}

function validatePrice($sKey, $sField, $sValue)
{
  $bError = false;
  if (substr($sValue, -2, -1) == "." || substr($sValue, -3, -2) == ".") $bError = true;
  if (substr_count($sValue, ",") > 1) $bError = true;
  if (ereg("[^0-9]", str_replace(",", "", str_replace(".", "", $sValue)))) $bError = true;

  if ($bError) return "Error: Wrong input format for price";
}

function convert2Date($sKey, $sField, $sValue)
{
  if ($sValue)
    return strftime("%d.%m.%Y", $sValue);
  else
    return "&nbsp;";
}

function convert2Empty($sKey, $sField, $sValue)
{
  if ($sValue == "")
    return "&nbsp;";
}

function callback($sCallbackName, $sKey, $aHash) {
  printf("Called Callback <b>%s</b> ", $sCallbackName);
  if ($sKey) printf("with key %s", $sKey);
  printf("<br>");
}

function callSave($sKey, $aHash) {
  callback("APL_ACTION_SAVE", $sKey, $aHash);
}

function callSaveNew($sKey, $aHash) {
  callback("APL_ACTION_SAVE_NEW", $sKey, $aHash);
}

function callCancel($sKey, $ahash) {
  callback("APL_ACTION_CANCEL", $sKey, $aHash);
}

function callRemove($sKey, $aHash) {
  callback("APL_ACTION_REMOVE", $sKey, $aHash);
}

function callAdd($sKey, $aHash) {
  callback("APL_ACTION_ADD", $sKey, $aHash);
}

function callModify($sKey, $aHash) {
  callback("APL_ACTION_MODIFY", $sKey, $aHash);
}

function callEventModBefore($sKey) {
  callback("APL_EVENT_MODIFY_BEFORE", $sKey, "");
}

function callEventModAfter($sKey) {
  callback("APL_EVENT_MODIFY_AFTER", $sKey, "");
}

// Detail Form

$Data["Product"] = new CProductControl($DB);
$Data["Product"]->setName("product");

$Group["Product"] = new CAPLGroupBox($Page);
$Group["Product"]->setCaption("Produktdetails");

$Form["Product"] = new CAPLDataForm($Group["Product"]);
$Form["Product"]->setDataControl($Data["Product"]);
$Form["Product"]->setActionHandler($Page->Action);
$Form["Product"]->setName("Form");

$Form["Product"]->setCallback(APL_ACTION_SAVE, "callSave");
$Form["Product"]->setCallback(APL_ACTION_SAVE_NEW, "callSaveNew");
$Form["Product"]->setCallback(APL_ACTION_CANCEL, "callCancel");
$Form["Product"]->setCallback(APL_ACTION_REMOVE, "callRemove");
$Form["Product"]->setCallback(APL_ACTION_MODIFY, "callModify");
$Form["Product"]->setCallback(APL_ACTION_ADD, "callAdd");


$Form["Product"]->Edit["ProductNo"] = new CAPLDataText($Form["Product"]);
$Form["Product"]->Edit["ProductNo"]->setName("productno");
$Form["Product"]->Edit["ProductNo"]->setDataControl($Data["Product"]);
$Form["Product"]->Edit["ProductNo"]->setCaption("Product No");
$Form["Product"]->Edit["ProductNo"]->setConvertor(APL_CONVERTOR_ACTION, "convert2Empty");

$Form["Product"]->Edit["Name"] = new CAPLDataEditBox($Form["Product"]);
$Form["Product"]->Edit["Name"]->setName("name");
$Form["Product"]->Edit["Name"]->setDataControl($Data["Product"]);
$Form["Product"]->Edit["Name"]->setCaption("Name");

$Form["Product"]->Edit["Name"]->setCallback(APL_EVENT_MODIFY_BEFORE, "callEventModBefore");
$Form["Product"]->Edit["Name"]->setCallback(APL_EVENT_MODIFY_AFTER, "callEventModAfter");

$Form["Product"]->Edit["Description"] = new CAPLDataTextBox($Form["Product"]);
$Form["Product"]->Edit["Description"]->setName("description");
$Form["Product"]->Edit["Description"]->setDataControl($Data["Product"]);
$Form["Product"]->Edit["Description"]->setCaption("Desciption");
$Form["Product"]->Edit["Description"]->setRows(3);
$Form["Product"]->Edit["Description"]->setCols(40);

$Form["Product"]->Edit["Sellingprice"] = new CAPLDataEditBox($Form["Product"]);
$Form["Product"]->Edit["Sellingprice"]->setName("sellingprice");
$Form["Product"]->Edit["Sellingprice"]->setDataControl($Data["Product"]);
$Form["Product"]->Edit["Sellingprice"]->setCaption("Selling price");
$Form["Product"]->Edit["Sellingprice"]->setConvertor(APL_CONVERTOR_ACTION, "convert2Price");
$Form["Product"]->Edit["Sellingprice"]->setConvertor(APL_CONVERTOR_STORAGE, "convert2Numeric");
$Form["Product"]->Edit["Sellingprice"]->setValidator(APL_VALIDATOR_STANDARD, "validatePrice", "");

$Form["Product"]->Check["Monthly"] = new CAPLDataCheckBox($Form["Product"]);
$Form["Product"]->Check["Monthly"]->setName("monthly");
$Form["Product"]->Check["Monthly"]->setDataControl($Data["Product"]);
$Form["Product"]->Check["Monthly"]->setCaption("Monthly");

$Form["Product"]->Edit["entrydate"] = new CAPLDataDateTime($Form["Product"]);
$Form["Product"]->Edit["entrydate"]->setName("entrydate");
$Form["Product"]->Edit["entrydate"]->setDataControl($Data["Product"]);
$Form["Product"]->Edit["entrydate"]->setCaption("Created on:");
$Form["Product"]->Edit["entrydate"]->setType(APL_DATETIME_DATE);

$Form["Product"]->Button["Save"] = new CAPLDataButton($Form["Product"]);
$Form["Product"]->Button["Save"]->setCaption("Save");
$Form["Product"]->Button["Save"]->setAction(APL_ACTION_SAVE);

$Form["Product"]->Button["SaveNew"] = new CAPLDataButton($Form["Product"]);
$Form["Product"]->Button["SaveNew"]->setCaption("Save New");
$Form["Product"]->Button["SaveNew"]->setAction(APL_ACTION_SAVE_NEW);

$Form["Product"]->Button["Delete"] = new CAPLDataButton($Form["Product"]);
$Form["Product"]->Button["Delete"]->setCaption("Remove");
$Form["Product"]->Button["Delete"]->setAction(APL_ACTION_REMOVE);

$Form["Product"]->Button["Cancel"] = new CAPLDataButton($Form["Product"]);
$Form["Product"]->Button["Cancel"]->setCaption("Cancel");
$Form["Product"]->Button["Cancel"]->setAction(APL_ACTION_CANCEL);

// Grid View

$Data["List"] = new CProductControl($DB);
$Data["List"]->setName("list");

$Group["List"] = new CAPLGroupBox($Page);
$Group["List"]->setCaption("Prooduktliste");

$Grid["List"] = new CAPLDataGrid($Group["List"]);
$Grid["List"]->setActionHandler($Page->Action);
$Grid["List"]->setDataControl($Data["List"]);
$Grid["List"]->setName("list");
$Grid["List"]->createFields(array("productno", "name", "entrydate", "monthly", "sellingprice"));
$Grid["List"]->setActionState(APL_ACTION_STANDARD, APL_STATE_VISIBLE);

$Grid["List"]->Field["productno"]->setCaption("No");
$Grid["List"]->Field["productno"]->setOrder();
$Grid["List"]->Field["name"]->setCaption("Name");
$Grid["List"]->Field["entrydate"]->setCaption("Created");
$Grid["List"]->Field["entrydate"]->setConvertor(APL_CONVERTOR_OUTPUT, "convert2Date");
$Grid["List"]->Field["sellingprice"]->setCaption("Sellingprice");
$Grid["List"]->Field["sellingprice"]->setConvertor(APL_CONVERTOR_OUTPUT, "convert2Price");

// Data Functions

function createProductNo($sKey)
{
  global $DB;
  
  if ($sKey) {
    $DB->execQuery(sprintf("UPDATE tblapldataform2product SET productno = '%s' WHERE productid = '%s'", $sKey, $sKey));
  }
}

// Register Event Callbacks

$Data["Product"]->setCallback(APL_EVENT_ADD_AFTER, "createProductNo");

// Action Evaluation

function showDetail($sKey, $aHash)
{
  global $DB, $Data, $Group;
  
  // Set Form Visible, Pass Key to Form Data
  $Group["Product"]->setState(APL_STATE_VISIBLE);
  $Data["Product"]->setPrimaryValue($sKey);
  
  // Set Grid Hidden
  $Group["List"]->setState(APL_STATE_HIDDEN);
}

// Register Callback

$Grid["List"]->setCallback(APL_ACTION_MODIFY_ASK, "showDetail");
$Grid["List"]->setCallback(APL_ACTION_ADD_ASK, "showDetail");
$Grid["List"]->setCallback(APL_ACTION_REMOVE_ASK, "showDetail");

// Action execution and page build

$Page->Action->execute();
$Page->draw();

?>
