<?php
/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: datagrid_callback.php,v 1.1 2001/12/05 13:44:48 ach Exp $ */

include("apl_config.php");

function dumpVars($sName, $aVars) {
  printf("Var Dump for <b>%s</b><br>", $sName);
  printf("<pre>");
  foreach ($aVars as $Item) {
    var_dump($Item);
  }
  printf("</pre>");
}  

class CDatabase extends CAPLDatabase
{
  function CDatabase()
  {
    CAPLDatabase::CAPLDatabase();
    $this->setHostname(DB_HOSTNAME);
    $this->setUsername(DB_USERNAME);
    $this->setPassword(DB_PASSWORD);
    $this->setDriver(DB_DRIVER);
    $this->setDatabase(DB_DATABASE); 
    $this->setType(DB_TYPE);
  }
}

class CUserSource extends CAPLDBControl
{  
  function CUserSource($Database)
  {
    CAPLDBControl::CAPLDBControl($Database);
    $this->setTable("tblaplcallbacks1users");
    $this->setPrimaryKey("userid");
    $this->setName("Users");
  }
}

class CGroupSource extends CAPLDBControl
{  
  function CGroupSource($Database)
  {
    CAPLDBControl::CAPLDBControl($Database);
    $this->setTable("tblaplcallbacks1groups");
    $this->setPrimaryKey("groupid");
    $this->setName("Groups");
  }
}

$Database = new CDatabase();
$UserSource = new CUserSource($Database);

$Action = new CAPLActionHandler;
$Page = new CAPLPage;

$GridUsers = new CAPLDataGrid($Page);
$GridUsers->setDataControl($UserSource);
$GridUsers->createFields(array("Name","Login"));
$GridUsers->setActionHandler($Action);
$GridUsers->setActionState(APL_ACTION_STANDARD, APL_STATE_VISIBLE);
$GridUsers->setActionState(APL_ACTION_MISC, APL_STATE_HIDDEN);
$GridUsers->setName("Users");

// CAPLDataControlObject Event

function callbackDataControl($key, $hash) {
  dumpVars("CAPLDBControl Callback", array($key, $hash));

  // return 1; // If return value > 0, Execution stops
}

$UserSource->setCallback(APL_EVENT_ADD_BEFORE, "callbackDataControl");
$UserSource->setCallback(APL_EVENT_MODIFY_AFTER, "callbackDataControl");

// CAPLDataObject Event

function callbackDataObject($key, $field, $value, $subnodes, $hash) {
  dumpVars("CAPLDataObject Callback", array($key, $field, $value, $subnodes, $hash));
  
  // return 1; // If return value > 0, Execution stops
}

$GridUsers->Field["Name"]->setCallback(APL_EVENT_ADD_BEFORE, "callbackDataObject");
$GridUsers->Field["Name"]->setCallback(APL_EVENT_MODIFY_BEFORE, "callbackDataObject");
$GridUsers->Field["Name"]->setCallback(APL_EVENT_REMOVE_BEFORE, "callbackDataObject");

// Validator

function callbackValidator($key, $field, $value, $hash) {
  dumpVars("CAPLDataObject Validator Standard", array($key, $field, $value, $hash));
  if ($value == "") return "Value was empty";
}

$GridUsers->Field["Name"]->setValidator(APL_VALIDATOR_STANDARD, "callbackValidator", "Error");

function callbackCollector($key, $field, $subvalues, $hash) {
  dumpVars("CAPLDataObject Validator Collector", array($key, $field, $subvalues, $hash));
}

$GridUsers->Field["Name"]->setValidator(APL_VALIDATOR_COLLECTOR, "callbackCollector", "Error");


// Failure Callback, fired in case a ValidatorCallback has a return Value

function callbackFailure($key, $field, $value, $hash) {
  dumpVars("CAPLDataGrid FailureCallback", array($key, $field, $value, $hash));
}

$GridUsers->setFailureCallback(APL_ACTION_STANDARD, "callbackFailure");


// Convertor

function callbackConvertorOutput($key, $field, $value, $hash) {
  dumpVars("CAPLDataObject Convertor Output", array($key, $field, $value, $hash));
  if ($value == "") return "Value was empty.";
}

function callbackConvertorAction($key, $field, $value, $hash) {
  dumpVars("CAPLDataObject Convertor Action", array($key, $field, $value, $hash));
  if ($value == "") return "Value was empty.";
}

$GridUsers->Field["Name"]->setConvertor(APL_CONVERTOR_OUTPUT, "callbackConvertorOutput");
$GridUsers->Field["Name"]->setConvertor(APL_CONVERTOR_ACTION, "callbackConvertorAction");

$Action->execute();
$Page->draw();

?>
