<?php
/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: datagrid_simple.php,v 1.1 2001/12/05 14:46:22 ach Exp $ */

include("apl_config.php");

// Global classes

class CDatabase extends CAPLDatabase
{
  function CDatabase()
  {
    CAPLDatabase::CAPLDatabase();
    $this->setHostname(DB_HOSTNAME);
    $this->setUsername(DB_USERNAME);
    $this->setPassword(DB_PASSWORD);
    $this->setDriver(DB_DRIVER);
    $this->setDatabase(DB_DATABASE); 
    $this->setType(DB_TYPE);
  }
}

class CProductSource extends CAPLDBControl
{  
  function CProductSource($Database)
  {
    CAPLDBControl::CAPLDBControl($Database);
    $this->setTable("tblapldatagrid1product");
    $this->setPrimaryKey("productid");
    $this->setName("products");
  }
}

// Global functions

function convert2Price($sKey, $sField, $sValue)
{
  if (!$sValue) $sValue = 0;
  return number_format($sValue, 2, ",",".");
}

function convert2Numeric($sKey, $sField, $sValue)
{
  echo $sValue;
  if ($sValue == "") return 0;
  $fValue = str_replace(",",".",str_replace(".","", $sValue));
  if ($fValue == "" || $fValue== ".") $fValue = 0;
  return $fValue;
}

function validatePrice($sKey, $sField, $sValue)
{
  $bError = false;
  if (substr($sValue, -2, -1) == "." || substr($sValue, -3, -2) == ".") $bError = true;
  if (substr_count($sValue, ",") > 1) $bError = true;
  if (ereg("[^0-9]", str_replace(",", "", str_replace(".", "", $sValue)))) $bError = true;

  if ($bError) return "Error: Wrong input format for price";
}

function convert2Date($sKey, $sField, $sValue)
{
  if ($sValue)
    return strftime("%d.%m.%Y", $sValue);
  else
    return "&nbsp;";
}

// Global Objects

$Database = new CDatabase();
$ActionHandler = new CAPLActionHandler();
$Page = new CAPLPage();

$Source["Grid"] = new CProductSource($Database);
$Source["Grid"]->setName("Grid");

$Grid = new CAPLDataGrid($Page);
$Grid->setActionHandler($ActionHandler);
$Grid->setDataControl($Source["Grid"]);
$Grid->createFields(array("Name", "Description", "entrydate", "sellingprice", "monthly"));
$Grid->setActionState(APL_ACTION_STANDARD, APL_STATE_VISIBLE);
$Grid->setName("Grid");

$Grid->Field["entrydate"]->setConvertor(APL_CONVERTOR_OUTPUT, "convert2Date");
$Grid->Field["sellingprice"]->setConvertor(APL_CONVERTOR_OUTPUT, "convert2Price");
$Grid->Field["sellingprice"]->setConvertor(APL_CONVERTOR_ACTION, "convert2Price");
$Grid->Field["sellingprice"]->setValidator(APL_VALIDATOR_STANDARD, "validatePrice", "Error: Wrong price format");
$Grid->Field["sellingprice"]->setConvertor(APL_CONVERTOR_STORAGE, "convert2Numeric");

// Execution & Page Build

$ActionHandler->execute();
$Page->draw();

?>
