/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldatacontrolobject.h,v 1.26 2002/03/29 22:51:25 ach Exp $ */


#ifndef APLDATAHANDLER_H
#define APLDATAHANDLER_H

#include <map>
#include <vector>
#include <string>

#include "aplcallback.h"
#include "apl_config.h"

typedef std::vector < std::string > keylist;
typedef std::map < std::string, std::string > dataset;
typedef std::map < std::string, dataset > datalist;

typedef struct {
  const std::string *Key;
  std::map < std::string, std::string > *Hash;
}
apl_datacontrol_event;

class CAPLDataObject;

class CAPLDataControlObject {
protected:
  std::vector < CAPLDataObject* > DataObjects;

  std::string sName;
  std::string sPrimaryValue;
  std::string sPrimaryKey;
  std::string sTable;

  int iRangeStart, iRangeSize;
  int iListSize;
  CAPLCallback event[EVENT_NUMBER];
  std::string filter;
  int filterType;
  std::string sError;
public:
  CAPLDataControlObject();
  virtual ~CAPLDataControlObject() {};

  // Identify the DataControlObject via a Name
  void addDataObject(CAPLDataObject* Child);
  std::vector < CAPLDataObject* > getDataObjects(void);

  // Primary Keys
  void setName(const std::string &);
  std::string getName();
  void setPrimaryKey(const std::string &, const std::string & = "");
  std::string getPrimaryKey(void);
  void setPrimaryValue(const std::string &);
  std::string getPrimaryValue(void);

  void setTable(const std::string &);
  std::string getTable(void);

  // Exchanging Data
  keylist KeyList;
  datalist DataList;
  keylist getList(void);
  dataset getData(const std::string & = "");
  void setData(const std::string &, dataset &, const bool = false);
  void addData(const std::string &, dataset &, const bool = false);
  void clearData(void);
  int getListSize(bool = false);
  
  // Error State
  void setError(const std::string &);
  std::string getError(void);

  // Limit & Range
  void setLimit(const int);
  void setRange(const int, const int);
  int getRangeStart(void);
  int getRangeSize(void);

  // Filtering
  void setFilter(const int, const std::string &);
  std::string getFilter(const int);

  // Event Callbacks
  void setCallback(int, apl_callback func, apl_userdat_dtor dtor, void *dat);
  int executeEvent(int, const std::string *, dataset *);

  // Virtual Functions for derived CAPLDataControlObject
  virtual void fetch(void) = 0;
  virtual std::string add(const std::string &, dataset &) = 0;
  virtual void remove(const std::string &) = 0;
  virtual std::string modify(const std::string &, dataset &) = 0;

  // Check
  void check(void);
};

#endif
