/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldatagrid.h,v 1.66 2002/04/01 22:28:25 ach Exp $ */


#ifndef APLDATAGRID_H
#define APLDATAGRID_H

#include <vector>
#include <string>
#include "apldataobject.h"
#include "apldatacontrolobject.h"
#include "aplengine.h"

#include "apldataform.h"
#include "apl_config.h"
#include "aplformobject.h"
#include "apleditbox.h"
#include "aplcombobox.h"
#include "aplcallback.h"
#include "aplquerystring.h"
#include "aplformcontrolobject.h"
#include "apldataformobject.h"

class CAPLDataGridField;
typedef std::vector < CAPLDataGridField* > fields;

class CAPLDataGrid : public CAPLFormControlObject {
protected:
  fields Fields;
  std::string align;
  std::string getFormHeader(void);
  std::string getFormData(void);
  std::string getFormFooter(void);
  std::string getFormHidden(const std::string &, const std::string &);
  std::string getHiddenSkip(void);
  std::string getHiddenFilter(void);
  std::string getHiddenOrder(void);
private:
  friend class CAPLDataGridField;
  std::string sMode;
  std::string actionKey;
  int actionMode;
  int actionState[ACTION_NUMBER];
  CAPLDataControlObject* DataControl;
  void executeAction(void);
  void prepareDraw(void);
  std::string outputGrid(void);
  void check(void);
  void makeActionKey(const std::string &);
  void evaluateAction(void);

  bool isActionGrid(void);
  int getRangeStart(void);
  int getRangeSize(void);
  int getActionMode(void);
  std::string getActionKey(void);
  dataset ActionHash;
  bool bMultipleOrder;
public:
  CAPLDataGrid(CAPLObject * = NULL);
  ~CAPLDataGrid() {};

  void addDataGridField(CAPLDataGridField *);
  void setDataControl(CAPLDataControlObject *);

  void execute(void);
  std::string output(void);
  void setAlign(const std::string &);
  void setActionState(const int, const int);
  int getActionState(const int);
  void setMultipleOrder(const bool = true);
  bool getMultipleOrder(void);
};

class CAPLDataGridField : public CAPLDataFormObject {
private:
  friend class CAPLDataGrid;
  CAPLDataGrid *DataGrid;
  CAPLFormObject *FormObject;
  CAPLComboBox *LookupObject;
  bool bDeleteFormObject;
  bool bDeleteLookupObject;
  std::string align, valign, height, width;
  std::string linkkey, linkpage, linkfieldkey;
  CAPLObject *OutputObject;
  bool bDeleteOutputObject;
  std::map < std::string, std::string > lookupData;
  bool haveLookupData;
  std::string lookupValue;
protected:
  int actionState[ACTION_NUMBER];
  std::string sCaption;
  void prepareDraw(void);
  std::string outputCaption(void);
  std::string outputListEntry(const std::string & = "");
  std::string outputListAction(const std::string & = "");
public:
  CAPLDataGridField(CAPLDataGrid* Parent);
  ~CAPLDataGridField();
  
  void setCaption(const std::string &);
  void setLink(const std::string &, const std::string &, const std::string & = "");
  void setLookupObject(CAPLComboBox *);
  void setActionObject(CAPLFormObject *);
  void setOutputObject(CAPLObject *);
  int executeRelayedCollector(apl_action_data *, CAPLActionHandler *);
  void setAlign(const std::string &);
  void setVAlign(const std::string &);
  void setWidth(const std::string &);
  void setHeight(const std::string &);
  void setActionState(const int, const int);
  int getActionState(const int);
};

#endif
