/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldataobject.h,v 1.43 2002/04/01 22:28:25 ach Exp $ */


#ifndef APLDATAOBJECT_H
#define APLDATAOBJECT_H

#include "apldatacontrolobject.h"
#include "aplcallback.h"
#include "aplactionhandler.h"
#include "apl_config.h"
#include <map>
#include <vector>

typedef struct {
  const std::string *key;
  const std::string *field;
  dataset *hash;
  std::string *errorMessage;
  int type;
  apl_action_data *subnode;
}
apl_dataobject_validator;

typedef struct {
  const std::string *key;
  const std::string *field;
  std::string *value;
  dataset *hash;
  bool keep;
}
apl_dataobject_convertor;

typedef struct {
  const std::string *key;
  const std::string *name;
  const std::string *field;
  const std::string *value;
  dataset *hash;
  apl_action_data *subnode;
}
apl_dataobject_event;

class CAPLDataLookup;

class CAPLDataObject : public CAPLObject {
public:
  CAPLDataObject(CAPLObject * = NULL);
  ~CAPLDataObject(void) {};

  void addLookup(CAPLDataLookup *DataLookup);
  virtual void setDataControl(CAPLDataControlObject *Parent);

  // Filters & Order
  void setFilter(const int, const std::string &);
  bool getFilter(void);
  std::string getFilter(const int);
  std::string filter[FILTER_NUMBER];
  void setOrder(const int = 0);
  int getOrder(void);

  // Callback, Validator, Convertor
  void setCallback(int, apl_callback, apl_userdat_dtor, void *data);
  int executeEvent(int, const std::string *, dataset *);
  void setValidator(int, apl_callback, apl_userdat_dtor, void *userdat, const std::string &);
  void executeValidator(int, const std::string *, dataset *, std::string &);
  void setConvertor(int, apl_callback, apl_userdat_dtor, void *userdat);
  void executeConvertor(int, const std::string *, std::string *, dataset *);

  virtual int executeRelayedCollector(apl_action_data *, CAPLActionHandler *) {
    return 0;
  };

  void setDataState(const int, const int);
  int getDataState(const int);
  std::string getValue(std::string = "");
  void check(const std::string & = "");

  CAPLDataLookup *DataLookup;
  CAPLDataControlObject *DataControl;
protected:
  CAPLCallback validator[VALIDATOR_NUMBER];
  CAPLCallback convertor[CONVERTOR_NUMBER];
  CAPLCallback event[EVENT_NUMBER];
  std::string errorMessage;
  int iOrder;
  bool bFilter;
  apl_action_data *subnode;

private:
  int datastate[DATACONTROL_NUMBER];
};

class CAPLDataLookup : public CAPLDataObject {
public:
  CAPLDataLookup(CAPLDataObject *Parent);
  ~CAPLDataLookup(void) {};
  
  std::string getData(const std::string &);
  keylist getList(void);
  void addData(const std::string &, const std::string & = "");
private:
  dataset Data;
  bool bFetched;
};

#endif
