/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplobject.h,v 1.24 2002/04/01 22:28:26 ach Exp $ */


#ifndef APLOBJECT_H
#define APLOBJECT_H

#include <vector>
#include <map>
#include <string>
#include <strutilsxx.h>
#include "aplengine.h"
#include "apl_config.h"

class CAPLObject;
typedef std::vector < CAPLObject* > children;
typedef std::map < std::string, CAPLObject* > slots;

class CAPLObject : public CAPLEngine {
private:
  children Children;
  slots Slots;
  CAPLObject* Parent;
  int state;
  std::string sValue;
  std::string sName;

protected:
  void prepareSlots(void);
  void prepareChildren(void);
  void prepare(void);

public:
  CAPLObject(CAPLObject * = NULL);
  virtual ~CAPLObject() {};

  CAPLObject* getChild(const unsigned int);
  CAPLObject* getParent(void);
  int getChildrenCount(void);

  void addChild(CAPLObject*);
  void addSlot(const std::string &, CAPLObject*);



  virtual void setState(const int);
  int getState(void);

  virtual void draw(void);
  virtual std::string output(void);
  virtual void execute(void) {};

  void setValue(const std::string &);
  std::string getValue(void);
  void setName(const std::string &);
  std::string getName(void);
  
  static std::string getUniqueName(void);
  static void initUniqueName(void);
};

#endif
