/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplops.h,v 1.18 2002/03/17 13:23:47 ach Exp $ */



#ifndef APLOPS_H
#define APLOPS_H

#include <string>
#include "aplexecutor.h"

class CAPLOp {
public:
CAPLOp() {};
  virtual ~CAPLOp() {};
  virtual std::string execute(CAPLExecutor *) = 0;
};

class CAPLOpOneArg : public CAPLOp {
public:
  CAPLOpOneArg(const char *data, size_t len);
protected:
  std::string data;
};

class CAPLOpVar : public CAPLOpOneArg {
public:
  CAPLOpVar(const char *data, size_t len) : CAPLOpOneArg(data, len) {};
  std::string execute(CAPLExecutor *);
};

class CAPLOpSlot : public CAPLOpOneArg {
public:
  CAPLOpSlot(const char *data, size_t len) : CAPLOpOneArg(data, len) {};
  std::string execute(CAPLExecutor *);
};

class CAPLOpIntern : public CAPLOpOneArg {
public:
  CAPLOpIntern(const char *data, size_t len) : CAPLOpOneArg(data, len) {};
  std::string execute(CAPLExecutor *);
};

class CAPLOpGlobal : public CAPLOpOneArg {
public:
  CAPLOpGlobal(const char *data, size_t len) : CAPLOpOneArg(data, len) {};
  std::string execute(CAPLExecutor *);
};

class CAPLOpChildren : public CAPLOp {
public:
  std::string execute(CAPLExecutor *);
};

class CAPLOpTheme : public CAPLOp {
public:
  std::string execute(CAPLExecutor *);
};

class CAPLOpThemeDir : public CAPLOp {
public:
  std::string execute(CAPLExecutor *);
};

class CAPLOpStyle : public CAPLOpOneArg {
public:
  CAPLOpStyle(const char *data, size_t len) : CAPLOpOneArg(data, len) {};
  std::string execute(CAPLExecutor *);
};

class CAPLOpStyleDir : public CAPLOp {
public:
  std::string execute(CAPLExecutor *);
};

class CAPLOpString : public CAPLOpOneArg {
public:
  CAPLOpString(const char *data, size_t len) : CAPLOpOneArg(data, len) {};
  std::string execute(CAPLExecutor *);
  void append(const char *text, size_t len);
};

class CAPLOpNewline : public CAPLOp {
public:
  CAPLOpNewline(size_t len) {
    count = len;
  };
  std::string execute(CAPLExecutor *);
private:
  size_t count;
};

#endif
