/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apltable.h,v 1.17 2002/04/01 22:28:26 ach Exp $ */


#ifndef APLTABLE_H
#define APLTABLE_H

#include <string>
#include "aplobject.h"

class CAPLTable: public CAPLObject {
private:
  void prepareDraw(void);
  std::string sWidth, sHeight;
  std::string sBackground;
  std::string sBgColor;
  int iCellPadding, iCellSpacing;
  int iBorder;
  bool compressEmptyCells;
  std::string align;
public:
  CAPLTable(CAPLObject * = NULL);
  virtual ~CAPLTable() {};
  
  std::string output();
  void setAlign(const std::string &);
  void setSize(const int, const int);
  void setWidth(const std::string &);
  void setHeight(const std::string &);
  void setBackground(const std::string &);
  void setBgColor(const std::string &);
  void setCellPadding(const int &);
  void setCellSpacing(const int &);
  void setBorder(const int &);
  void setCompressEmptyCells(const bool);
  bool getCompressEmptyCells();
};

class CAPLTableRow: public CAPLObject {
  friend class CAPLTable;
private:
  std::string sBgColor;
  std::string sAlign;
  std::string sVAlign;
public:
  CAPLTableRow(CAPLObject* Parent);
  virtual ~CAPLTableRow() {};
  
  void setBgColor(const std::string &);
  void setAlign(const std::string &);
  void setVAlign(const std::string &);
  std::string output();
};

class CAPLTableCell: public CAPLObject {
  friend class CAPLTable;
private:
  std::string sBgColor;
  std::string sBackground;
  std::string sAlign;
  std::string sVAlign;
  std::string sWidth;
  std::string sHeight;
  int iColspan;
  int iRowspan;
  bool bHidden;
public:
  CAPLTableCell(CAPLObject* Parent);
  virtual ~CAPLTableCell() {};
  void setBgColor(const std::string &);
  void setBackground(const std::string &);
  void setAlign(const std::string &);
  void setVAlign(const std::string &);
  void setWidth(const std::string &);
  void setHeight(const std::string &);
  void setColspan(const int);
  void setRowspan(const int);
  std::string output();
};

#endif
