/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplbutton.cpp,v 1.33 2002/09/24 15:04:21 k Exp $ */


#include "aplbutton.h"

using namespace std;
using namespace strutilsxx;

CAPLButton::CAPLButton(CAPLObject* Parent) : CAPLFormObject(Parent) {
  setType(BUTTON_STANDARD);
  setTemplate("button");
  sHeight = "";
  sWidth = "";
  iBorder = -1;
}

void CAPLButton::setType(const int type) {
  this->type = type;
}

void CAPLButton::setImage(const string &imgSrc, const string &imgAlt) {
  this->type = BUTTON_IMAGE;
  this->imgSrc = imgSrc;
  this->imgAlt = imgAlt;
}

void CAPLButton::setWidth(const string &sWidth) {
  this->sWidth = sWidth;
}

void CAPLButton::setHeight(const string &sHeight) {
  this->sHeight = sHeight;
}

void CAPLButton::setBorder(const int iBorder) {
  this->iBorder = iBorder;
}

string CAPLButton::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";

    prepare();
    setIntern("Name", getName());

    switch (type) {
    case BUTTON_STANDARD:
      setIntern("Type", "submit");
      break;
    case BUTTON_IMAGE:
      setIntern("Type", "image");
      break;
    case BUTTON_RESET:
      setIntern("Type", "reset");
      break;
    }

    if (sCaption != "" && getValue() == "") setValue(sCaption);
    if (getValue() != "") sParams += "value=\"" + getValue() + "\"";
    if (imgSrc != "") sParams += " src=\"" + imgSrc + "\"";
    if (imgAlt != "") sParams += " alt=\"" + imgAlt + "\"";
    if (sHeight != "") sParams += " height=\"" + sHeight + "\"";
    if (sWidth != "") sParams += " width=\"" + sWidth + "\"";
    if (iBorder != -1) sParams += " border=\"" + intstr(iBorder) + "\"";
    if (!(getState() & STATE_ENABLED)) sParams += " disabled=\"disabled\"";

    setIntern("Optional", sParams);
    output += outputTemplate();
  }
  return output;
}
