/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplcheckbox.cpp,v 1.32 2002/09/24 15:04:21 k Exp $ */


#include "aplcheckbox.h"
#include "aplhiddenfield.h"

using namespace std;

CAPLCheckBox::CAPLCheckBox(CAPLObject* Parent) : CAPLFormObject(Parent) {
  setChecked(false);
  setTemplate("checkbox");
  setType(CHECKBOX_STANDARD);
}

void CAPLCheckBox::setChecked(const bool &bChecked) {
  this->bChecked = bChecked;
}

void CAPLCheckBox::setType(const int &iType) {
  this->iType = iType; 
}

string CAPLCheckBox::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";
    prepare();
    
    if (sCaption != "") {
      setIntern("Caption", sCaption);
      setIntern("Caption", outputTemplate("caption"));
    }
    setIntern("Name", getName());

    if (this->iType == CHECKBOX_BOOLEAN) {
      setIntern("Value", "1");

      // Checked Status depends on value, empty value is considered new
      if (getValue() == "0")
        this->setChecked(false);
      else if (getValue() == "1")
        this->setChecked(true);
        
      // Hidden Field for ZERO Value submission
      CAPLHiddenField* Hidden = new CAPLHiddenField();
      Hidden->setName(getName());
      Hidden->setValue("0");
      output += Hidden->output();
      delete Hidden;
    } else {
      setIntern("Value", getValue());
    }
    
    if (bChecked) sParams += "checked=\"checked\"";
    if (!(getState() & STATE_READWRITE)) sParams += " readonly=\"readonly\"";
    if (!(getState() & STATE_ENABLED)) sParams += " disabled=\"disabled\"";

    setIntern("Optional", sParams);
    output += outputTemplate();
  }
  return output;
}

