/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplcombobox.cpp,v 1.37 2002/09/24 15:04:21 k Exp $ */


#include "aplcombobox.h"

using namespace strutilsxx;
using namespace std;

CAPLComboBox::CAPLComboBox(CAPLObject* Parent) : CAPLFormObject(Parent) {
  bMultiple = false;
  iSize = 0;
  compare = 0;
  setTemplate("combobox");
}

void CAPLComboBox::addComboBoxItem(CAPLComboBoxItem* Item) {
  this->Item.push_back(Item);
}

void CAPLComboBox::setSize(const int iNewSize) {
  iSize = iNewSize;
}

void CAPLComboBox::setMultiple(const bool &bMultiple) {
  this->bMultiple = bMultiple;
}

void CAPLComboBox::addItem(const string &sNewValue, const string &sNewText, bool bSelected) {
  CAPLComboBoxItem * newItem;

  newItem = new CAPLComboBoxItem(this);
  newItem->setValue(sNewValue);
  newItem->setText(sNewText);
  newItem->setSelected(bSelected);
}

void CAPLComboBox::setCompare(const int &type) {
  this->compare = type;
}

string CAPLComboBox::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";

    setIntern("Name", getName());
    if (sCaption != "") {
      setIntern("Caption", sCaption);
      setIntern("Caption", outputTemplate("caption"));
    }
    if (iSize) sParams = " size=\"" + intstr(iSize) + "\"";
    if (bMultiple) sParams += " multiple=\"multiple\"";
    if (!(getState() & STATE_ENABLED)) sParams += " disabled=\"disabled\"";
    setIntern("Optional", sParams);

    for (unsigned int i = 0; i < Item.size(); i++) {

      if (compare == 0) {
        if (Item[i]->getValue() == getValue()) Item[i]->setSelected();
      } else if (compare == 1) {
        if (Item[i]->getText() == getValue()) Item[i]->setSelected();
      }
      Item[i]->setTemplateType(getTemplateType());
      output += Item[i]->output();
    }
    setChildren(output);
    output = outputTemplate();
  }
  return output;
}

CAPLComboBoxItem::CAPLComboBoxItem(CAPLComboBox *Parent) {
  Parent->addComboBoxItem(this);
  bSelected = false;
  sText = "";
  setTemplate("combobox");
  this->setTemplate(Parent->getTemplate(), Parent->getAlternativeTemplate());
}

void CAPLComboBoxItem::setSelected(const bool bNewSelected) {
  bSelected = bNewSelected;
}

void CAPLComboBoxItem::setText(const string &sNewText) {
  sText = sNewText;
}

string CAPLComboBoxItem::getText(void) {
  return sText;
}

string CAPLComboBoxItem::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";

    if (sText == "")
      setIntern("Text", getValue());
    else
      setIntern("Text", sText);
    setIntern("Value", getValue());
    if (bSelected) sParams += " selected=\"selected\"";
    if (!(getState() & STATE_ENABLED)) sParams += " disabled=\"disabled\"";
    setIntern("Optional", sParams);

    output += outputTemplate("item");
  }
  return output;
}
