/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldatacontrolobject.cpp,v 1.30 2002/09/24 15:04:21 k Exp $ */


#include <stdexcept>
#include "apldatacontrolobject.h"
#include "apldataobject.h"

using namespace std;

CAPLDataControlObject::CAPLDataControlObject() {
  iRangeStart = 0;
  iRangeSize = -1;
  iListSize = 0;
  filter = "";
  sPrimaryValue = "";
  sPrimaryKey = "";
  sError = "";
  this->setName(CAPLObject::getUniqueName());
}

void CAPLDataControlObject::addDataObject(CAPLDataObject* Child) {
  DataObjects.push_back(Child);
}

vector < CAPLDataObject * > CAPLDataControlObject::getDataObjects(void) {
  return DataObjects;
}

void CAPLDataControlObject::setName(const string &sNewName) {
  this->sName = sNewName;
}

string CAPLDataControlObject::getName(void) {
  return sName;
}

void CAPLDataControlObject::setLimit(const int iRangeSize) {
  iRangeStart = 0;
  this->iRangeSize = iRangeSize;
}

void CAPLDataControlObject::setRange(const int iRangeStart, const int iRangeSize) {
  this->iRangeStart = iRangeStart;
  this->iRangeSize = iRangeSize;
}

int CAPLDataControlObject::getRangeStart(void) {
  return iRangeStart;
}

int CAPLDataControlObject::getRangeSize(void) {
  return iRangeSize;
}

void CAPLDataControlObject::setPrimaryKey(const string &sPrimaryKey, const string &sPrimaryValue) {
  this->sPrimaryKey = sPrimaryKey;
  if (sPrimaryValue != "") this->sPrimaryValue = sPrimaryValue;
}

string CAPLDataControlObject::getPrimaryKey(void) {
  return sPrimaryKey;
}

void CAPLDataControlObject::setPrimaryValue(const string &sPrimaryValue) {
  this->sPrimaryValue = sPrimaryValue;
}

string CAPLDataControlObject::getPrimaryValue(void) {
  return sPrimaryValue;
}

void CAPLDataControlObject::setTable(const string &table) {
  this->sTable = table;
}

string CAPLDataControlObject::getTable(void) {
  return sTable;
}

keylist CAPLDataControlObject::getList(void) {
  return KeyList;
}

dataset CAPLDataControlObject::getData(const string &primaryKey) {
  if (getListSize() && primaryKey.length()) {
    return DataList[primaryKey];
  } else {
    dataset hash;
    return hash;
  }
}

void CAPLDataControlObject::setData(const string &primaryKey, dataset &DataSet, const bool bExplicit) {
  if (primaryKey.length()) {
    if (bExplicit) {
      DataList[primaryKey] = DataSet;
    } else {
      for (unsigned int i = 0; i < DataObjects.size(); i++) {
        if (DataObjects[i]->getDataState(DATACONTROL_UPDATE) & STATE_ENABLED) {
          DataList[primaryKey][DataObjects[i]->getName()] = DataSet[DataObjects[i]->getName()];
        }
      }
    }
  }
}

void CAPLDataControlObject::addData(const string &primaryKey, dataset &DataSet, const bool bExplicit) {
  if (primaryKey.length()) {
    KeyList.push_back(primaryKey);
    setData(primaryKey, DataSet, bExplicit);
  }
}

void CAPLDataControlObject::clearData(void) {
  DataList.clear();
  KeyList.clear();
}

int CAPLDataControlObject::getListSize(bool fullSize) {
  if (fullSize)
    return iListSize;
  else
    return KeyList.size();
}

void CAPLDataControlObject::setCallback(int type, apl_callback func, apl_userdat_dtor dtor, void *dat) {
  if (type < 0 || type >= EVENT_NUMBER)
    throw runtime_error("Event type specifier exceeds limit");

  event[type].setCallback(func, dtor, dat);
}

int CAPLDataControlObject::executeEvent(int type, const string *Key, dataset *Hash) {
  int iNewID = 0;
  int state = DATACONTROL_ADD;
  apl_datacontrol_event evdat;

  if (type == EVENT_ADD_BEFORE || type == EVENT_ADD_AFTER) state = DATACONTROL_ADD;
  if (type == EVENT_MODIFY_BEFORE || type == EVENT_MODIFY_AFTER) state = DATACONTROL_MODIFY;
  if (type == EVENT_REMOVE_BEFORE || type == EVENT_REMOVE_AFTER) state = DATACONTROL_REMOVE;

  for (unsigned int i = 0; i < DataObjects.size(); i++) {
    if (DataObjects[i]->getDataState(state) & STATE_ENABLED) {
      iNewID = DataObjects[i]->executeEvent(type, Key, Hash);
      if (iNewID) return iNewID;
    }
  }

  evdat.Key = Key;
  evdat.Hash = Hash;

  return event[type].execute(&evdat);
}

void CAPLDataControlObject::setFilter(const int type, const string &filter) {
  this->filter = filter;
}

string CAPLDataControlObject::getFilter(const int type) {
  return filter;
}

void CAPLDataControlObject::check(void) {
  if (!getPrimaryKey().length()) throw runtime_error("PrimaryKey was not defined for object of type CAPLDataControlObject");
  if (!getName().length()) throw runtime_error("Name was not defined for object of type CAPLDataControlObject");
  for (unsigned int i = 0; i < DataObjects.size(); i++) DataObjects[i]->check();
}

void CAPLDataControlObject::setError(const string &sError) {
  this->sError = sError;
}

string CAPLDataControlObject::getError(void) {
  return this->sError;
}
