/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldataform.cpp,v 1.87 2002/09/24 15:04:21 k Exp $ */


#include "apldataform.h"

using namespace strutilsxx;
using namespace std;

CAPLDataForm::CAPLDataForm(CAPLObject* Parent) : CAPLFormControlObject(Parent) {
  enctype = "";
  setTemplate("dataform", "form");
  setFailure(false);
}

void CAPLDataForm::setDataControl(CAPLDataControlObject *Parent) {
  addDataControl(Parent);
}

void CAPLDataForm::addDataControl(CAPLDataControlObject *Parent) {
  if (Parent->getName() == "") Parent->setName("form" + intstr(DataControls.size()));
  DataControls.push_back(Parent);
}

int CAPLDataForm::getActionMode(void) {
  if (isActionForm()) {
    if (ActionHandler->existsKey("APLFormSave_x") || ActionHandler->existsKey("APLFormSave"))
      return ACTION_SAVE;
    if (ActionHandler->existsKey("APLFormSaveNew_x") || ActionHandler->existsKey("APLFormSaveNew"))
      return ACTION_SAVE_NEW;
    if (ActionHandler->existsKey("APLFormCancel_x") || ActionHandler->existsKey("APLFormCancel"))
      return ACTION_CANCEL;
    if (ActionHandler->existsKey("APLFormRemove_x") || ActionHandler->existsKey("APLFormRemove"))
      return ACTION_REMOVE;      
  }
  return -1;
}

bool CAPLDataForm::isActionForm(void) {
  if (ActionHandler->getData("APLFormName").value == getName()) return true;
  return false;
}

void CAPLDataForm::executeAction(void) {
  int mode = getActionMode();

  if (isActionForm()) {
    string error;
    string field;
    for (unsigned int i = 0; i < DataControls.size(); i++) {
      int mode2 = -1;
      string sDCname = DataControls[i]->getName();
      unsigned int objSize = DataControls[i]->getDataObjects().size();
      map < string, string > Hash;
      string sPrimaryValue = ActionHandler->data["APLPrimary"].data[sDCname].value;

      switch(mode) {
        case ACTION_CANCEL:
          executeEvent(mode, &sPrimaryValue, NULL);
          break;
        case ACTION_REMOVE:
          if (sPrimaryValue.length()) {
            executeEvent(mode, &sPrimaryValue, NULL);
            DataControls[i]->remove(sPrimaryValue);
          }
          break;  
        case ACTION_SAVE:
        case ACTION_SAVE_NEW:

        // We have two action modes here

        if (sPrimaryValue != "" && sPrimaryValue != "-1")
          mode2 = ACTION_MODIFY;
        else
          mode2 = ACTION_ADD;

        // Collect Data

        for (unsigned int z = 0; z < objSize; z++) {
          field = DataControls[i]->getDataObjects()[z]->getName();

          if (ActionHandler->data["Data"].data[sDCname].data.count(field)) {
            DataControls[i]->getDataObjects()[z]->executeRelayedCollector(&ActionHandler->data["Data"].data[sDCname].data[field], ActionHandler);
            Hash[field] = ActionHandler->data["Data"].data[sDCname].data[field].value;
          } else {
            if (mode2 == ACTION_ADD && DataControls[i]->getDataObjects()[z]->getValue().length()) {
              Hash[field] = DataControls[i]->getDataObjects()[z]->getValue();
            } else {
              DataControls[i]->getDataObjects()[z]->setDataState(DATACONTROL_UPDATE, STATE_DISABLED);
            }
          }
        }
        ActionHash[i] = Hash;       

        // Execute Validator Collector

        for (unsigned int z = 0; z < objSize; z++) {
          string field = DataControls[i]->getDataObjects()[z]->getName();
          DataControls[i]->getDataObjects()[z]->executeValidator(VALIDATOR_COLLECTOR, &sPrimaryValue, &Hash, error);

          if (error.length()) {
            executeFailureCallback(error, sPrimaryValue, field, Hash[field], Hash, mode2);
            sMode[i] = sPrimaryValue;
            return ;
          }
        }

        // Execute Validator

        for (unsigned int z = 0; z < objSize; z++) {
          string field = DataControls[i]->getDataObjects()[z]->getName();
          DataControls[i]->getDataObjects()[z]->executeValidator(VALIDATOR_STANDARD, &sPrimaryValue, &Hash, error);

          if (error.length()) {
            executeFailureCallback(error, sPrimaryValue, field, Hash[field], Hash, mode2);
            sMode[i] = sPrimaryValue;
            return ;
          }
        }

        // Execute Convertor

        for (unsigned int z = 0; z < objSize; z++) {
          DataControls[i]->getDataObjects()[z]->executeConvertor(CONVERTOR_STORAGE, &sPrimaryValue, &(Hash[DataControls[i]->getDataObjects()[z]->getName()]), &Hash);
        }

        // Data Exchange

        if (mode2 == ACTION_MODIFY) {
          DataControls[i]->modify(sPrimaryValue, Hash);
          executeEvent(mode2, &sPrimaryValue, &Hash);
        } else {
          sPrimaryValue = DataControls[i]->add("", Hash);
          executeEvent(mode2, &sPrimaryValue, &Hash);
        }
        
        if (mode == ACTION_SAVE)
          executeEvent(mode, &sPrimaryValue, &Hash);
        else if (mode == ACTION_SAVE_NEW)
          executeEvent(mode, NULL, NULL);
          
        if (mode == ACTION_SAVE) DataControls[i]->setPrimaryValue(sPrimaryValue);
        break;
      }
    }
  }
}

string CAPLDataForm::getFormHidden(const string &name, const string &value) {
  CAPLHiddenField Hidden;

  Hidden.setName(name);
  Hidden.setValue(value);
  return Hidden.output();
}

string CAPLDataForm::outputForm() {
  map < string, string > ActionVars;
  string output = "";

  setTemplateType(templateType);

  setIntern("Method", "post");
  setIntern("Action", ActionHandler->getPage());
  if (enctype != "") setIntern("Optional", "enctype=\"" + enctype + "\"");

  for (unsigned int i = 0; i < DataControls.size(); i++) {
    if (this->getFailure()) {
      if (sMode[i] == "") sMode[i] = "-1";
      DataControls[i]->setPrimaryValue(sMode[i]);
    }
    DataControls[i]->fetch();

    // If an Error occurred, write old data back to form inputs
    if (this->getFailure()) {
      if (sMode[i] == "-1")
        DataControls[i]->addData(sMode[i], ActionHash[i]);
      else
        DataControls[i]->setData(sMode[i], ActionHash[i]);
      DataControls[i]->setError(sMode[i]);
    }

    // Output ActionKey
    output += getFormHidden("APLPrimary[" + DataControls[i]->getName() + "]", DataControls[i]->getPrimaryValue());
  }

  // ActionVars

  ActionVars = ActionHandler->getVars();
  for (map < string, string > ::iterator Iterate = ActionVars.begin(); Iterate != ActionVars.end(); Iterate++) {
    output += getFormHidden((*Iterate).first, (*Iterate).second);
  }
  output += getFormHidden("APLFormName", getName());
  prepareSlots();
  output += CAPLObject::output();
  setChildren(output);
  output = outputTemplate();

  return output;
}

void CAPLDataForm::check() {
  if (ActionHandler == NULL) throw runtime_error("ActionHandler was not defined for object of type CAPLDataForm");
  if (getName() == "") throw runtime_error("Name was not defined for object of type CAPLDataForm");
}

void CAPLDataForm::execute() {
  APL_TRACEIN("CAPLDataForm::execute");

  check();
  executeAction();

  APL_TRACEOUT("CAPLDataForm::execute");
}

string CAPLDataForm::output() {
  APL_TRACEIN("CAPLDataForm::output");
  string output = "";

  check();
  if (getState() & STATE_VISIBLE) output = outputForm();

  APL_TRACEOUT("CAPLDataForm::output");
  return output;
}

void CAPLDataForm::setEnctype(const string &enctype) {
  this->enctype = enctype;
}
