/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldatagrid.cpp,v 1.121 2002/09/24 15:04:21 k Exp $ */

#include "apldatagrid.h"

using namespace strutilsxx;
using namespace std;

CAPLDataGrid::CAPLDataGrid(CAPLObject* Parent) : CAPLFormControlObject(Parent) {
  align = "";
  sMode = "";
  actionKey = "";
  actionMode = -1;
  bMultipleOrder = false;

  setTemplate("datagrid", "grid");
  setActionState(ACTION_STANDARD, STATE_HIDDEN);
  setActionState(ACTION_MISC, STATE_VISIBLE);
}

bool CAPLDataGrid::isActionGrid(void) {
  if (getName() == ActionHandler->getData("APLGridName").value)
    return true;
  else
    return false;
}

void CAPLDataGrid::makeActionKey(const string &mode) {
  actionKey = ActionHandler->getData(mode).getKey();
}

int CAPLDataGrid::getActionMode(void) {
  return actionMode;
}

void CAPLDataGrid::evaluateAction(void) {
  if (isActionGrid()) {
    
    // Collect Filters and Orders
    
    for (unsigned int i = 0; i < Fields.size(); i++) {
      string field = Fields[i]->getName();
      string sCurrent = "";


      string order = ActionHandler->data["APLOrder"].data[field].value;
      if (Fields[i]->DataLookup)
        Fields[i]->DataLookup->setOrder(atoi(order.c_str()));
      else
        Fields[i]->setOrder(atoi(order.c_str()));
      
      string wildcard = ActionHandler->getData("APLFilter").data[field].value;
      if (Fields[i]->DataLookup)
        Fields[i]->DataLookup->setFilter(FILTER_WILDCARD, wildcard);
      else
        Fields[i]->setFilter(FILTER_WILDCARD, wildcard);
    }

    // Evaluate Action Mode

    if (ActionHandler->existsKey("APLGridAdd")) {
      actionMode = ACTION_ADD;
      return ;
    }

    if (ActionHandler->existsKey("APLGridAddAsk")) {
      actionMode = ACTION_ADD_ASK;
      return ;
    }

    if (ActionHandler->existsKey("APLGridModify")) {
      makeActionKey("APLGridModify");
      actionMode = ACTION_MODIFY;
      return ;
    }

    if (ActionHandler->existsKey("APLGridModifyAsk")) {
      makeActionKey("APLGridModifyAsk");
      actionMode = ACTION_MODIFY_ASK;
      return ;
    }

    if (ActionHandler->existsKey("APLGridRemove")) {
      makeActionKey("APLGridRemove");
      actionMode = ACTION_REMOVE;
      return ;
    }

    if (ActionHandler->existsKey("APLGridRemoveAsk")) {
      makeActionKey("APLGridRemoveAsk");
      actionMode = ACTION_REMOVE_ASK;
      return ;
    }

    if (ActionHandler->existsKey("APLGridSkipNext") || ActionHandler->existsKey("APLGridSkipPrevious")) {
      actionMode = ACTION_SKIP;
      return ;
    }

    if (ActionHandler->existsKey("APLGridFilter")) {
      apl_action_hash tmpFields = ActionHandler->getData("APLGridFilter").data;

      for (unsigned int i = 0; i < Fields.size(); i++) {
        string tmpField = Fields[i]->getName();
        string tmp = "";
        string tmpFilter = tmpFields[tmpField].value;

        if (Fields[i]->DataLookup)
          tmp = Fields[i]->DataLookup->getFilter(FILTER_WILDCARD);
        else
          tmp = Fields[i]->getFilter(FILTER_WILDCARD);

        if (tmpFilter != tmp) {
          if (Fields[i]->DataLookup)
            Fields[i]->DataLookup->setFilter(FILTER_WILDCARD, tmpFilter);
          else
            Fields[i]->setFilter(FILTER_WILDCARD, tmpFilter);
          actionMode = ACTION_FILTER;
          return ;
        }
      }
    }

    if (ActionHandler->existsKey("APLGridCancel")) {
      actionMode = ACTION_CANCEL;
      return ;
    }

    if (ActionHandler->existsKey("APLGridOrder")) {
      string tmpOrder = ActionHandler->getData("APLGridOrder").getKey();

      for (unsigned int i = 0; i < Fields.size(); i++) {
        if (tmpOrder == Fields[i]->getName()) {
          int newOrder;
          int tmp;
          
          if (Fields[i]->DataLookup)
            tmp = Fields[i]->DataLookup->getOrder();
          else
            tmp = Fields[i]->getOrder();

          switch (tmp) {
            case 0:
              newOrder = 1;
              break;
            case 1:
              newOrder = -1;
              break;
            default:
              newOrder = 0;
          }
          if (Fields[i]->DataLookup)
            Fields[i]->DataLookup->setOrder(newOrder);
          else
            Fields[i]->setOrder(newOrder);
            
          if (!bMultipleOrder) {
            for (unsigned int z = 0; z < Fields.size(); z++) {
              if (Fields[z]->getName() != Fields[i]->getName()) {
                if (Fields[z]->DataLookup)
                  Fields[z]->DataLookup->setOrder(-1);
                else
                  Fields[z]->setOrder(-1);
              }
            }
          }
          actionMode = ACTION_ORDER;
          return ;
        }
      }
    }
  }
}

string CAPLDataGrid::getActionKey(void) {
  return actionKey;
}

int CAPLDataGrid::getRangeStart(void) {
  if (isActionGrid() && getActionMode() != ACTION_FILTER && getActionMode() != ACTION_ORDER) {
    int iTmpRangeStart = 0;

    if (ActionHandler->existsKey("APLGridRangeStart"))
      iTmpRangeStart = atoi(ActionHandler->getData("APLGridRangeStart").value.c_str());

    if (ActionHandler->existsKey("APLGridSkipNext"))
      iTmpRangeStart = iTmpRangeStart + DataControl->getRangeSize();

    if (ActionHandler->existsKey("APLGridSkipPrevious"))
      iTmpRangeStart = iTmpRangeStart - DataControl->getRangeSize();

    return iTmpRangeStart;
  } else
    return DataControl->getRangeStart();
}

int CAPLDataGrid::getRangeSize(void) {
  return DataControl->getRangeSize();
}

void CAPLDataGrid::setAlign(const string &align) {
  this->align = align;
}

void CAPLDataGrid::addDataGridField(CAPLDataGridField* Field) {
  Fields.push_back(Field);
}

void CAPLDataGrid::setDataControl(CAPLDataControlObject* newDataControl) {
  DataControl = newDataControl;
}

void CAPLDataGrid::executeAction(void) {
  // Set DataControl for all Fields
  for (unsigned int i = 0; i < Fields.size(); i++) Fields[i]->setDataControl(DataControl);
  
  // Parse Submitted Values
  evaluateAction();

  // Execute Actions depending on key and mode
  int mode = getActionMode();
  string key = getActionKey();
  dataset * tmpHash = NULL;
  string * tmpKey = NULL;

  switch (mode) {
  case ACTION_ADD:
  case ACTION_MODIFY:
    {
      dataset Hash;
      string sError;
      unsigned int objSize = Fields.size();

      // Execute Collector - collect Form Values and put them into a dataset

      for (unsigned int i = 0; i < objSize; i++) {
        string field = Fields[i]->getName();
        cerr << "Field Name = " << field << endl;
        if (Fields[i]->getState() & STATE_VISIBLE) {
          if (ActionHandler->data["Data"].data.count(field)) {
            Fields[i]->executeRelayedCollector(&ActionHandler->data["Data"].data[field], ActionHandler);
            Hash[field] = ActionHandler->data["Data"].data[field].value;
          } else {
            if (mode == ACTION_ADD && Fields[i]->getValue().length()) {
              Hash[field] = Fields[i]->getValue();
            } else {
              Fields[i]->setDataState(DATACONTROL_UPDATE, STATE_DISABLED);
            }
          }
        } else {
          if (mode == ACTION_ADD) {
            cerr << "Field Value = " << Fields[i]->getValue() << endl;
            Hash[field] = Fields[i]->getValue();
          } else {
            Fields[i]->setDataState(DATACONTROL_UPDATE, STATE_DISABLED);
          }
        }
      }
      
      ActionHash = Hash;

      // Execute Validator Collector and display Error Message

      for (unsigned int i = 0; i < objSize; i++) {
        Fields[i]->executeValidator(VALIDATOR_COLLECTOR, &key, &Hash, sError);
        string field = Fields[i]->getName();

        if (sError.length()) {
          executeFailureCallback(sError, key, field, Hash[field], Hash, mode);
          sMode = key;
          return ;
        }
      }

      // Execute Validator and display Error Message

      for (unsigned int i = 0; i < objSize; i++) {
        Fields[i]->executeValidator(VALIDATOR_STANDARD, &key, &Hash, sError);
        string field = Fields[i]->getName();

        if (sError.length()) {
          executeFailureCallback(sError, key, field, Hash[field], Hash, mode);
          sMode = key;
          return ;
        }
      }

      for (unsigned int i =0; i < objSize; i++) {
        Fields[i]->executeConvertor(CONVERTOR_STORAGE, &key, &Hash[Fields[i]->getName()], &Hash);
      }

      // Data Exchange: Modify & Add

      if (mode == ACTION_MODIFY) {
        tmpKey = &key;
        DataControl->modify(key, Hash);
      } else {
        DataControl->add("", Hash);
      }
      tmpHash = &Hash;

      break;
    }
  case ACTION_REMOVE:
    if (key != "") {
      tmpKey = &key;
      DataControl->remove(key);
    }
    break;
  case ACTION_MODIFY_ASK:
    tmpKey = &key;
    break;
  case ACTION_REMOVE_ASK:
    tmpKey = &key;
    break;
  }

  // Execute Actions

  if (mode >= 0 && mode < ACTION_NUMBER) {
    executeEvent(mode, tmpKey, tmpHash);

    if (mode == ACTION_MODIFY_ASK || mode == ACTION_MODIFY || mode == ACTION_ADD_ASK || mode == ACTION_ADD ||
        mode == ACTION_REMOVE_ASK || mode == ACTION_REMOVE || mode == ACTION_CANCEL)
      executeEvent(ACTION_STANDARD, tmpKey, tmpHash);

    if (mode == ACTION_FILTER || mode == ACTION_ORDER || mode == ACTION_SKIP)
      executeEvent(ACTION_MISC, tmpKey, tmpHash);
  }
}

void CAPLDataGrid::prepareDraw(void) {
  for (unsigned int i = 0; i < Fields.size(); i++) {
    Fields[i]->prepareDraw();
  }
  DataControl->setRange(getRangeStart(), getRangeSize());
  DataControl->fetch();
}

string CAPLDataGrid::getFormHeader(void) {
  string sReturn = "";

  setIntern("Action", ActionHandler->getPage());
  setIntern("Method", "post");
  sReturn += outputTemplate("form/header");

  return sReturn;
}

string CAPLDataGrid::getFormData(void) {
  map < string, string > ActionVars;
  string sReturn = getFormHidden("APLGridName", getName());

  ActionVars = ActionHandler->getVars();
  for (map < string, string > ::iterator Iterate = ActionVars.begin(); Iterate != ActionVars.end(); Iterate++) {
    sReturn += getFormHidden((*Iterate).first, (*Iterate).second);
  }

  return sReturn;
}

string CAPLDataGrid::getFormFooter(void) {
  string sReturn = outputTemplate("form/footer");
  return sReturn;
}

string CAPLDataGrid::getFormHidden(const string &sName, const string &sValue) {
  CAPLHiddenField Hidden;

  Hidden.setName(sName);
  Hidden.setValue(sValue);

  return Hidden.output();
}

string CAPLDataGrid::getHiddenFilter(void) {
  string sTmp = "";

  for (unsigned int i = 0; i < Fields.size(); i++) {
    string tmp = "";
    if (Fields[i]->DataLookup)
      tmp = Fields[i]->DataLookup->getFilter(FILTER_WILDCARD);
    else
      tmp = Fields[i]->getFilter(FILTER_WILDCARD);
    if (tmp.length())
      sTmp += getFormHidden("APLFilter[" + Fields[i]->getName() + "]", tmp);
  }
  return sTmp;
}

string CAPLDataGrid::getHiddenOrder(void) {
  string sTmp = "";
  string sCurrent = "";

  for (unsigned int i = 0; i < Fields.size(); i++) {
    int tmp;
    if (Fields[i]->DataLookup)
      tmp = Fields[i]->DataLookup->getOrder();
    else
      tmp = Fields[i]->getOrder();

    sTmp += getFormHidden("APLOrder[" + Fields[i]->getName() + "]", intstr(tmp));
  }
  return sTmp;
}

string CAPLDataGrid::getHiddenSkip(void) {
  string sTmp = "";

  sTmp += getFormHidden("APLGridRangeStart", intstr(getRangeStart()));
  sTmp += getFormHidden("APLGridRangeSize", intstr(DataControl->getRangeSize()));
  return sTmp;
}

string CAPLDataGrid::outputGrid() {
  APL_TRACEIN("CAPLDataGrid::outputGrid");

  // Get the form Data
  
  if (this->getFailure()) {
    if (sMode.length())
      DataControl->setData(sMode, ActionHash);
    else {
      sMode = "-1";
      DataControl->addData(sMode, ActionHash);
    }
    DataControl->setError(sMode);
  }

  // Start Grid Output
  
  keylist::size_type iList;
  keylist PrimaryKeys = DataControl->getList();
  string sSkipNext = "", sSkipPrevious = "";
  int iMode = getActionMode(), iOldMode = -1;
  int fieldCount = Fields.size();
  string output = "";
  string tmp = "";

  setTemplateType(templateType);
  if (align != "") setIntern("Optional", "align=\"" + align + "\"");

  if (getActionState(ACTION_MODIFY) & STATE_VISIBLE ||
      getActionState(ACTION_ADD) & STATE_VISIBLE ||
      getActionState(ACTION_REMOVE) & STATE_VISIBLE) tmp = outputTemplate("header/action");

  for (unsigned int i = 0; i < Fields.size(); i++) tmp += Fields[i]->outputCaption();
  setChildren(tmp);

  output += outputTemplate("header/main");

  // From now on we process the body

  for (iList = 0; iList < PrimaryKeys.size(); iList++) {
    string outputBody = "";
    string primaryValue = PrimaryKeys[iList];

    if (iMode == ACTION_MODIFY && primaryValue == sMode) {
      iOldMode = ACTION_MODIFY;
      iMode = ACTION_MODIFY_ASK;
    } else {
      if (iOldMode != -1) {
        iMode = iOldMode;
        iOldMode = -1;
      }
    }

    if (primaryValue != "-1") {
      if (getActionKey() == primaryValue && (iMode == ACTION_MODIFY_ASK || iMode == ACTION_REMOVE_ASK)) {
        string sTmp = "";

        if (iMode == ACTION_MODIFY_ASK)
          setIntern("Name", "APLGridModify[" + primaryValue + "]");
        else
          setIntern("Name", "APLGridRemove[" + primaryValue + "]");
        sTmp += outputTemplate("action/confirm");
        setIntern("Name", "APLGridCancel[]");
        sTmp += outputTemplate("action/cancel");

        setIntern("Action", sTmp);
        outputBody += outputTemplate("body/action");

        for (unsigned int i = 0; i < Fields.size(); i++) {
          if (iMode == ACTION_MODIFY_ASK)
            outputBody += Fields[i]->outputListAction(primaryValue);
          else
            outputBody += Fields[i]->outputListEntry(primaryValue);
        }
      } else {
        if (getActionState(ACTION_MODIFY) & STATE_VISIBLE || getActionState(ACTION_REMOVE) & STATE_VISIBLE || getActionState(ACTION_ADD) & STATE_VISIBLE) {
          string sTmp = "";
          if (getActionState(ACTION_MODIFY) & STATE_VISIBLE) {
            setIntern("Name", "APLGridModifyAsk[" + primaryValue + "]");
            sTmp += outputTemplate("action/modify");
          }
          if (getActionState(ACTION_REMOVE) & STATE_VISIBLE) {
            setIntern("Name", "APLGridRemoveAsk[" + primaryValue + "]");
            sTmp += outputTemplate("action/remove");
          }
          setIntern("Action", sTmp);
          outputBody += outputTemplate("body/action");
        }
        for (unsigned int i = 0; i < Fields.size(); i++) {
          outputBody += Fields[i]->outputListEntry(primaryValue);
        }
      }
      setChildren(outputBody);
      output += outputTemplate("body/main");
    }
  }

  // Body Section for ACTION_ADD_ASK modus

  string outputBody = "";
  if (iMode == ACTION_ADD && sMode == "-1") iMode = ACTION_ADD_ASK;

  if (iMode == ACTION_ADD_ASK) {
    string primaryValue = "";
    string sTmp = "";

    if (sMode == "-1") primaryValue = sMode;

    setIntern("Name", "APLGridAdd[]");
    sTmp += outputTemplate("action/confirm");
    setIntern("Name", "APLGridCancel[]");
    sTmp += outputTemplate("action/cancel");

    setIntern("Action", sTmp);
    outputBody += outputTemplate("body/action");

    for (unsigned int i = 0; i < Fields.size(); i++) outputBody += Fields[i]->outputListAction(primaryValue);

    setChildren(outputBody);
    output += outputTemplate("body/main");
  }

  // Footer section / Skip Add Button

  string outputFooter = "";

  if (getActionState(ACTION_ADD) & STATE_VISIBLE) {
    setIntern("Name", "APLGridAddAsk[]");
    string sTmp = outputTemplate("action/add");
    setIntern("Action", sTmp);
  } else {
    setIntern("Action", "");
  }

  if (getActionState(ACTION_MODIFY) & STATE_VISIBLE ||
      getActionState(ACTION_ADD) & STATE_VISIBLE ||
      getActionState(ACTION_REMOVE) & STATE_VISIBLE) outputFooter += outputTemplate("footer/action");

  if ((getActionState(ACTION_SKIP) & STATE_VISIBLE)) {
    if (DataControl->getListSize(1) > getRangeSize() && getRangeSize() != -1) {
      if (getRangeStart() > 0) {
        setIntern("Name", "APLGridSkipPrevious[]");
        sSkipPrevious = outputTemplate("footer/skipprevious");
      }
      if ((getRangeStart() + getRangeSize()) < DataControl->getListSize(1) && DataControl->getListSize(1) > 1) {
        setIntern("Name", "APLGridSkipNext[]");
        sSkipNext = outputTemplate("footer/skipnext");
      }
    }
    setIntern("Previous", sSkipPrevious);
    setIntern("Next", sSkipNext);
  }

  setIntern("Colspan", intstr(fieldCount));
  setIntern("Skip", outputTemplate("footer/skip"));
  outputFooter += outputTemplate("footer/data");


  setChildren(outputFooter);
  output += outputTemplate("footer/main");

  setChildren(output);

  output = "";
  output += getFormHeader();
  output += getFormData();
  output += getHiddenFilter();
  output += getHiddenOrder();
  output += getHiddenSkip();
  output += outputTemplate();
  output += getFormFooter();

  APL_TRACEOUT("CAPLDataGrid::outputGrid");
  return output;
}

void CAPLDataGrid::check() {
  if (ActionHandler == NULL) throw runtime_error("ActionHandler was not defined for object of type CAPLDataGrid");
  if (DataControl == NULL) throw runtime_error("DataControl was not defined for object of type CAPLDataGrid");
  if (getName() == "") throw runtime_error("Name was not defined for object of type CAPLDataGrid");
}

void CAPLDataGrid::execute() {
  APL_TRACEIN("CAPLDataGrid::execute");

  check();
  executeAction();

  APL_TRACEOUT("CAPLDataGrid::execute");
}

string CAPLDataGrid::output() {
  APL_TRACEIN("CAPLDataGrid::output");
  string output = "";

  check();
  if (getState() & STATE_VISIBLE) {
    prepareDraw();
    output += outputGrid();
  }

  APL_TRACEOUT("CAPLDataGrid::output");
  return output;
}

int CAPLDataGrid::getActionState(const int identifier) {
  return actionState[identifier];
}

void CAPLDataGrid::setActionState(const int identifier, const int state) {
  if (identifier == ACTION_STANDARD) {
    setActionState(ACTION_ADD, state);
    setActionState(ACTION_MODIFY, state);
    setActionState(ACTION_REMOVE, state);
    setActionState(ACTION_SKIP, state);
  }

  if (identifier == ACTION_MISC) {
    setActionState(ACTION_FILTER, state);
    setActionState(ACTION_ORDER, state);
    setActionState(ACTION_SKIP, state);
  }

  if (identifier == ACTION_FILTER || identifier == ACTION_ORDER) {
    for (unsigned int i = 0; i < Fields.size(); i++) Fields[i]->setActionState(identifier, state);
  }

  if (state & 1) {
    actionState[identifier] &= (~state);
  } else {
    actionState[identifier] |= state;
  }
}

void CAPLDataGrid::setMultipleOrder(const bool bMultipleOrder) {
  this->bMultipleOrder = bMultipleOrder;
}

bool CAPLDataGrid::getMultipleOrder(void) {
  return this->bMultipleOrder;
}

CAPLDataGridField::CAPLDataGridField(CAPLDataGrid* Parent) {
  Parent->addDataGridField(this);
  DataGrid = Parent;

  // Passed Objects
  FormObject = NULL;
  LookupObject = NULL;
  OutputObject = NULL;
  bDeleteFormObject = false;
  bDeleteLookupObject = false;
  bDeleteOutputObject = false;

  // Appearance
  align = "";
  valign = "";
  width = "";
  height = "";

  // Link on a field
  linkpage = "";
  linkkey = "";
  linkfieldkey = "";

  haveLookupData = false;

  setTemplate("datagrid", "grid");
  setTemplateType(Parent->getTemplateType());
  setActionState(ACTION_MISC, STATE_VISIBLE);
}

CAPLDataGridField::~CAPLDataGridField() {
  if (bDeleteFormObject) delete FormObject;
  if (bDeleteLookupObject) delete LookupObject;
}

int CAPLDataGridField::getActionState(const int identifier) {
  return actionState[identifier];
}

void CAPLDataGridField::setActionState(const int identifier, const int state) {
  if (identifier == ACTION_MISC) {
    setActionState(ACTION_FILTER, state);
    setActionState(ACTION_ORDER, state);
  }

  if (state & 1) {
    actionState[identifier] &= (~state);
  } else {
    actionState[identifier] |= state;
  }
}

void CAPLDataGridField::prepareDraw(void) {}


void CAPLDataGridField::setCaption(const string &sCaption) {
  this->sCaption = sCaption;
}

string CAPLDataGridField::outputCaption(void) {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sTmpCaption = "", sTmpFilter = "", sTmpOrder = "";
    string option = "";

    if (getActionState(ACTION_ORDER) & STATE_VISIBLE) {
      string sTmpOrderButton = "";
      string newID = "";
      string tmpTmpl = "";
      int tmpOrder;
      if (DataLookup)
        tmpOrder = DataLookup->getOrder();
      else
        tmpOrder = getOrder();

      switch (tmpOrder) {
      case 1:
        tmpTmpl = "header/orderup";
        break;
      case 0:
        tmpTmpl = "header/orderdown";
        break;
      case - 1:
        tmpTmpl = "header/orderdefault";
        break;
      }
      setIntern("Name", "APLGridOrder[" + getName() + "]");
      sTmpOrderButton += outputTemplate(tmpTmpl);

      setIntern("Order", sTmpOrderButton);
      sTmpOrder += outputTemplate("header/order");
    }

    if (getActionState(ACTION_FILTER) & STATE_VISIBLE) {
      string tmpFilter = "";
      if (DataLookup)
        tmpFilter = DataLookup->getFilter(FILTER_WILDCARD);
      else
        tmpFilter = getFilter(FILTER_WILDCARD);

      setIntern("Name", "APLGridFilter[" + getName() + "]");
      setIntern("Value", tmpFilter);
      sTmpFilter += outputTemplate("header/filter");
    }

    setIntern("Caption", sCaption);
    sTmpCaption = outputTemplate("header/caption");

    if (width != "") option += "width=\"" + width + "\" ";
    setIntern("Optional", option);
    setIntern("Caption", sTmpCaption);
    setIntern("Order", sTmpOrder);
    setIntern("Filter", sTmpFilter);
    output += outputTemplate("header/data");
  }
  return output;
}

string CAPLDataGridField::outputListEntry(const string &primaryKey) {
  APL_TRACEIN("CAPLDataGridField::outputListEntry");
  string output = "";

  if (getState() & STATE_VISIBLE) {
    string sShowValue = getValue(primaryKey);
    executeConvertor(CONVERTOR_OUTPUT, &primaryKey, &sShowValue, &DataControl->DataList[primaryKey]);
    string sOption = "";

    if (height != "") sOption += "height=\"" + height + "\" ";
    if (align != "") sOption += "align=\"" + align + "\" ";
    if (valign != "") sOption += "valign=\"" + valign + "\" ";
    setIntern("Optional", sOption);

    output += outputTemplate("body/dataheader");

    if (!sShowValue.empty()) {
      // If an output Object was specified, the value is passed to it
      if (OutputObject != NULL) OutputObject->setValue(sShowValue);

      // If the field has an link, the values are passed to the Link Object
      if (linkpage != "") {
        CAPLQueryString QueryString;

          // Append the actionvars to the query
          map < string, string > ActionVars = DataGrid->ActionHandler->getVars();
          for (map < string, string > ::iterator Iterate = ActionVars.begin(); Iterate != ActionVars.end(); Iterate++) {
            if ((*Iterate).first.substr(0,5) != "APLtf" && (*Iterate).first.substr(0,5) != "APLtm") {
              QueryString.setQueryVar((*Iterate).first, (*Iterate).second);
            }
          }
        QueryString.setQueryVar(linkkey, primaryKey);
        if (linkfieldkey != "") QueryString.setQueryVar(linkfieldkey, sShowValue);

        setIntern("Link", QueryString.getQueryString(linkpage));
        output += outputTemplate("body/linkheader");
      }

      // Either draw the output object or draw the template
      if (OutputObject != NULL)
        output += OutputObject->output();
      else {
        setIntern("Value", sShowValue);
        output += outputTemplate("body/datavalue");
      }

      // Draw the link footer
      if (linkpage != "") output += outputTemplate("body/linkfooter");
    } else {
      output += outputTemplate("body/dataempty");
    }
    output += outputTemplate("body/datafooter");
  }
  APL_TRACEOUT("CAPLDataGridField::outputListEntry");
  return output;
}

string CAPLDataGridField::outputListAction(const string &primaryKey) {
  APL_TRACEIN("CAPLDataGridField::outputListAction");
  string output = "";

  if (getState() & STATE_VISIBLE) {
    string sOption = "";
    string sShowValue = getValue(primaryKey);
    executeConvertor(CONVERTOR_ACTION, &primaryKey, &sShowValue, &DataControl->DataList[primaryKey]);

    if (height != "") sOption += "height=\"" + height + "\" ";
    if (align != "") sOption += "align=\"" + align + "\" ";
    if (valign != "") sOption += "valign=\"" + valign + "\" ";
    setIntern("Optional", sOption);

    output += outputTemplate("body/dataheader");
    if (DataLookup != NULL) {

      if (FormObject) {
        FormObject->setValue(sShowValue);
        FormObject->setName("Data[" + getName() + "]");
        FormObject->setTemplateType(templateType);
        output += FormObject->output();
      } else {
        if (LookupObject == NULL) {
          LookupObject = new CAPLComboBox();
          bDeleteLookupObject = true;
        }

        keylist List = DataLookup->getList();

        for (unsigned int i = 0; i < List.size(); i++) {
          LookupObject->addItem(List[i], DataLookup->getData(List[i]));
        }

        LookupObject->setValue(sShowValue);
        if (DataControl->getError() != primaryKey) LookupObject->setCompare(1);
        LookupObject->setName("Data[" + getName() + "]");
        LookupObject->setTemplateType(templateType);
        output += LookupObject->output();
      }
    } else {
      if (FormObject == NULL) {
        FormObject = new CAPLEditBox();
        bDeleteFormObject = true;
      }
      FormObject->setValue(sShowValue);
      FormObject->setName("Data[" + getName() + "]");
      FormObject->setTemplateType(templateType);
      output += FormObject->output();
    }
    output += outputTemplate("body/datafooter");
  }
  APL_TRACEOUT("CAPLDataGridField::outputListAction");
  return output;
}

void CAPLDataGridField::setLookupObject(CAPLComboBox* ComboBox) {
  if (bDeleteLookupObject) {
    delete this->LookupObject;
    bDeleteLookupObject = false;
  }
  this->LookupObject = ComboBox;
}

void CAPLDataGridField::setActionObject(CAPLFormObject* FormObject) {
  if (bDeleteFormObject) {
    delete this->FormObject;
    bDeleteFormObject = false;
  }
  this->FormObject = FormObject;
}

void CAPLDataGridField::setOutputObject(CAPLObject* OutputObject) {
  if (bDeleteOutputObject) {
    delete this->OutputObject;
    bDeleteOutputObject = false;
  }
  this->OutputObject = OutputObject;
}

int CAPLDataGridField::executeRelayedCollector(apl_action_data *data, CAPLActionHandler *handler) {
  if (FormObject)
    return FormObject->executeCollector(data, handler);
  return 0;
}

void CAPLDataGridField::setAlign(const string &align) {
  this->align = align;
}

void CAPLDataGridField::setVAlign(const string &valign) {
  this->valign = valign;
}

void CAPLDataGridField::setWidth(const string &width) {
  this->width = width;
}

void CAPLDataGridField::setHeight(const string &height) {
  this->height = height;
}

void CAPLDataGridField::setLink(const string &page, const string &key, const string &fieldKey) {
  this->linkpage = page;
  this->linkkey = key;
  this->linkfieldkey = fieldKey;
}
