/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldataset.cpp,v 1.21 2002/09/24 15:04:21 k Exp $ */


#include "apldataset.h"

using namespace std;

CAPLDataSet::CAPLDataSet(void) {
  fetched = false;
  DataControl = NULL;
}

void CAPLDataSet::addDataSetField(CAPLDataSetField* DataSetField) {
  check();
  DataSetFields.push_back(DataSetField);
  DataSetField->setDataControl(DataControl);
}

void CAPLDataSet::setDataControl(CAPLDataControlObject* DataControl) {
  this->DataControl = DataControl;
}

void CAPLDataSet::check() {
  if (DataControl == NULL) throw runtime_error("DataControl was not defined for object of type CAPLDataSet");
}

void CAPLDataSet::fetch(void) {
  check();
  DataControl->fetch();
  fetched = true;
}

keylist CAPLDataSet::getList(void) {
  if (!fetched) fetch();
  return DataControl->getList();
}

int CAPLDataSet::getListSize(void) {
  if (!fetched) fetch();
  return DataControl->getListSize();
}

dataset CAPLDataSet::getData(const string &key) {
  if (!fetched) fetch();
  return DataControl->getData(key);
}

string CAPLDataSet::add(const string &key, dataset &hash) {
  check();
  return DataControl->add(key, hash);
}

void CAPLDataSet::remove(const string &key) {
  check();
  DataControl->remove(key);
}

string CAPLDataSet::modify(const string &key, dataset &hash) {
  check();
  return DataControl->modify(key, hash);
}

CAPLDataSetField::CAPLDataSetField(CAPLDataSet* DataSet) {
  APL_TRACEIN("CAPLDataSetField::CAPLDataSetField");
  DataSet->addDataSetField(this);
  APL_TRACEOUT("CAPLDataSetField::CAPLDataSetField");
}

