/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldatetime.cpp,v 1.29 2002/09/24 15:04:21 k Exp $ */


#include "apldatetime.h"
#include "aplcheckbox.h"
#include <time.h>

using namespace strutilsxx;
using namespace std;

CAPLDateTime::CAPLDateTime(CAPLObject* Parent) : CAPLFormObject(Parent) {
  yearBegin = yearEnd = 10;
  yearLimitBegin = yearLimitEnd = -1;
 
  setTemplate("datetime");
  type = DATETIME_STANDARD;
  bCheckBox = false;
  setCollector(&CAPLDateTime::makeTimestamp, NULL, &bCheckBox);
}

int CAPLDateTime::makeTimestamp(void *checkbox, void *collectData) {
  struct tm *ta;
  time_t t;
  
  // Casting void parameters
  apl_formobject_collector *data = (apl_formobject_collector *) collectData;
  bool *bCheckBox = (bool *) checkbox;

  if (*bCheckBox == false || data->actionData->data["CheckBox"].value != "") {
    // Fill Default with current Date
    t = time(NULL);
    ta = localtime(&t);
    
    // Put values
    if (data->actionData->data["Hour"].value != "") ta->tm_hour = atoi(data->actionData->data["Hour"].value.c_str());
    if (data->actionData->data["Minute"].value != "") ta->tm_min = atoi(data->actionData->data["Minute"].value.c_str());
    if (data->actionData->data["Second"].value != "") ta->tm_sec = atoi(data->actionData->data["Second"].value.c_str());
    if (data->actionData->data["Day"].value != "") ta->tm_mday = atoi(data->actionData->data["Day"].value.c_str());
    if (data->actionData->data["Month"].value != "") ta->tm_mon = atoi(data->actionData->data["Month"].value.c_str()) - 1;
    if (data->actionData->data["Year"].value != "") ta->tm_year = atoi(data->actionData->data["Year"].value.c_str()) - 1900;
    ta->tm_wday = 0;
    ta->tm_isdst = -1;

    t = mktime(ta);
    data->actionData->value = intstr(t);
  } else {
    data->actionData->value = "";
  } 
  return 0;
}

string CAPLDateTime::getDateTimeBox(const int itype, const string &name, const int &start, const int &stop, const int &selected) {
  string sReturn = "";
  CAPLEngine * Engine;
  string stype;

  switch (itype) {
  case DATETIME_TIME:
    stype = "time";
    break;
  case DATETIME_DATE:
    stype = "date";
    break;
  }

  Engine = new CAPLEngine();
  Engine->setTemplate(this->getTemplate(), this->getAlternativeTemplate());
  Engine->setTemplateType(getTemplateType());

  for (int i = start; i < stop + 1; i++) {
    string sParams = "";

    Engine->setIntern("Text", intstr(i));
    Engine->setIntern("Value", intstr(i));
    if (i == selected) sParams += " selected=\"selected\"";
    Engine->setIntern("Optional", sParams);
    sReturn += Engine->outputTemplate(stype + "/combobox/item");
  }
  Engine->setChildren(sReturn);

  Engine->setIntern("Name", name);
  if (!(getState() & STATE_ENABLED))
    Engine->setIntern("Optional", "disabled=\"disabled\"");
  else
    Engine->setIntern("Optional", "");
  sReturn = Engine->outputTemplate(stype + "/combobox/main");

  delete Engine;
  return sReturn ;
}

void CAPLDateTime::setYearRange(const int &start, const int &stop) {
  yearBegin = start;
  if (stop == -1)
    yearEnd = start;
  else
    yearEnd = stop;
}

void CAPLDateTime::setYearLimit(const int &start, const int &stop) {
  yearLimitBegin = start;
  yearLimitEnd = stop;
}

void CAPLDateTime::setType(const int type) {
  this->type = type;
}

void CAPLDateTime::setCheckBox(const bool &bCheckBox) {
  this->bCheckBox = bCheckBox;
}

string CAPLDateTime::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";
    string tmp = "";
    time_t t;
    tm *ta;

    prepare();

    // If no value was set, we assume timestamp as current time()
    if (getValue().length() > 0 && getValue() != "0")
      t = atoi(getValue().c_str());
    else
      time(&t);
      
    // Time Struct gets the value
    ta = localtime(&t);

    if (sCaption != "") {
      setIntern("Caption", sCaption);
      setIntern("Caption", outputTemplate("caption"));
    }

    if (this->bCheckBox) {
      CAPLEngine * Engine = new CAPLEngine();
      Engine->setTemplate(this->getTemplate(), this->getAlternativeTemplate());
      Engine->setIntern("Name", getName() + "[CheckBox]");
      Engine->setIntern("Value", "1");
      if ((getValue().length() > 0 && getValue() != "0"))
        Engine->setIntern("Optional", "checked=\"checked\"");
      else
        Engine->setIntern("Optional", "");
      setIntern("CheckBox", Engine->outputTemplate("checkbox"));
    }

    if (type == DATETIME_STANDARD || type == DATETIME_TIME) {
      setIntern("Separator", outputTemplate("time/separator"));
      setIntern("Hour", getDateTimeBox(DATETIME_TIME, getName() + "[Hour]", 0, 23, ta->tm_hour));
      setIntern("Minute", getDateTimeBox(DATETIME_TIME, getName() + "[Minute]", 0, 59, ta->tm_min));
      setIntern("Second", getDateTimeBox(DATETIME_TIME, getName() + "[Second]", 0, 59, ta->tm_sec));
      setIntern("Time", outputTemplate("time/main"));
    }

    if (type == DATETIME_STANDARD || type == DATETIME_DATE) {
      setIntern("Separator", outputTemplate("date/separator"));
      setIntern("Day", getDateTimeBox(DATETIME_DATE, getName() + "[Day]", 1, 31, ta->tm_mday));
      setIntern("Month", getDateTimeBox(DATETIME_DATE, getName() + "[Month]", 1, 12, ta->tm_mon + 1));

      int tmpStart = ta->tm_year - yearBegin + 1900;
      int tmpStop = ta->tm_year + yearEnd + 1900;
      if (yearLimitBegin > 0) {
        tmpStart = yearLimitBegin;
        tmpStop = yearLimitEnd;
      }

      setIntern("Year", getDateTimeBox(DATETIME_DATE, getName() + "[Year]", tmpStart, tmpStop, ta->tm_year + 1900));
      setIntern("Date", outputTemplate("date/main"));
    }

    if (type == DATETIME_STANDARD)
      setIntern("Separator", outputTemplate("separator"));
    else
      setIntern("Separator", "");

    output += outputTemplate();
  }
  return output;
}
