/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apldbcontrol.cpp,v 1.51 2002/04/01 22:28:26 ach Exp $ */


#include "apldbcontrol.h"
#include "aplengine.h"

using namespace strutilsxx;
using namespace std;

CAPLDBControl::CAPLDBControl(CAPLDatabase *Database) {
  this->Database = Database;
  bufferSize = 1024;
}

void CAPLDBControl::setBufferSize(const int &bufferSize) {
  this->bufferSize = bufferSize;
}

string CAPLDBControl::getFullName(CAPLDataObject *DataObject) {
  return DataObject->DataControl->getName() + "." + DataObject->getName();
}

string CAPLDBControl::getRealName(CAPLDataObject *DataObject) {
  if (DataObject->DataLookup)
    return getFullName(DataObject->DataLookup);
  else
    return getFullName(DataObject);
}

void CAPLDBControl::fetch(void) {
  APL_TRACEIN("CAPLDBControl::fetch");

  check();
  clearData();
  bool handleDB = Database->isConnected();

  if (getTable() != "" && getPrimaryKey() != "") {
    string sQuery = "", sCountHead = "", sMainHead = "";
    CAPLDBResult *Result = NULL;
    int z = 0;  // Indicates whether to use AND or WHERE in creation of the SQL Query
    bool hasFilter = false;  // If any filter is set, a PrimaryValue is set from result

    // Create SQL Query
    sMainHead = "SELECT " + getName() + "." + getPrimaryKey();
    sCountHead = "SELECT COUNT(" + getName() + "." + getPrimaryKey() + ")";

    for (unsigned int i = 0; i < DataObjects.size(); i++) {
      if (DataObjects[i]->getDataState(DATACONTROL_FETCH) & STATE_ENABLED) {
        sMainHead += ", " + getRealName(DataObjects[i]);
      }
    }

    sQuery += " FROM " + getTable() + " AS " + getName();

    for (unsigned int i = 0; i < DataObjects.size(); i++) {
      if (DataObjects[i]->getDataState(DATACONTROL_FETCH) & STATE_ENABLED) {
        if (DataObjects[i]->DataLookup) {
          sQuery += " LEFT JOIN " + DataObjects[i]->DataLookup->DataControl->getTable();
          sQuery += " AS " + DataObjects[i]->DataLookup->DataControl->getName();
          sQuery += " ON " + getFullName(DataObjects[i]) + "=";
          sQuery += DataObjects[i]->DataLookup->DataControl->getName() + "." + DataObjects[i]->DataLookup->DataControl->getPrimaryKey();
        }
      }
    }

    // Select special primary value
    if (getPrimaryValue() != "") {
      sQuery += " WHERE " + getName() + "." + sPrimaryKey + "='" + sPrimaryValue + "'";
      z++;
    }

    // Add the Global Filter
    if (filter != "") {
      if (z)
        sQuery += " AND ";
      else
        sQuery += " WHERE ";
      sQuery += filter;
      z++;
    }

    // Add the filters to Query
    for (unsigned int i = 0; i < DataObjects.size(); i++) {
      if (DataObjects[i]->getDataState(DATACONTROL_FETCH) & STATE_ENABLED) {
        for (int y = 0; y < FILTER_NUMBER; y++) {
          switch (y) {
          case FILTER_WILDCARD:
            if (DataObjects[i]->getFilter(FILTER_WILDCARD).length()) {
              if (z)
                sQuery += " AND ";
              else
                sQuery += " WHERE ";
              sQuery += getFullName(DataObjects[i]) + " LIKE '" + replace(DataObjects[i]->filter[y], "*", "%") + "'";
              z++;
            }
            if (DataObjects[i]->DataLookup) {
              if (DataObjects[i]->DataLookup->getFilter(FILTER_WILDCARD).length()) {
                if (z)
                  sQuery += " AND ";
                else
                  sQuery += " WHERE ";
                sQuery += getFullName(DataObjects[i]->DataLookup) + " LIKE '" + replace(DataObjects[i]->DataLookup->filter[y], "*", "%") + "'";
                z++;
              }
            }
            break;
          case FILTER_STANDARD:
            if (DataObjects[i]->getFilter(FILTER_STANDARD).length()) {
              if (z)
                sQuery += " AND ";
              else
                sQuery += " WHERE ";
              sQuery += replace(DataObjects[i]->filter[y], "#", getFullName(DataObjects[i]));
              z++;
            }
            if (DataObjects[i]->DataLookup) {
              if (DataObjects[i]->DataLookup->getFilter(FILTER_STANDARD).length()) {
                if (z)
                  sQuery += " AND ";
                else
                  sQuery += " WHERE ";
                sQuery += replace(DataObjects[i]->DataLookup->filter[y], "#", getFullName(DataObjects[i]->DataLookup));
                z++;
              }
            }
            break;
          }
        }
      }
    }

    APL_TRACEVAR("CAPLDBControl::fetch", "Query", sQuery);

    try {
      if (!handleDB) Database->connect();
      Result = (CAPLDBResult *) Database->openQuery(sCountHead + sQuery, bufferSize);
      if (Result->fetch()) {
        iListSize = Result->getInt(0);
      }
      Database->closeQuery(Result);
      if (!handleDB) Database->disconnect();
    }
    catch (apldatabase_error E) {
      throw runtime_error(E.what());
    }

    if (z) hasFilter = true;

    //Add the orders to Query
    z = 0;
    for (unsigned int i = 0; i < DataObjects.size(); i++) {
      if (DataObjects[i]->getDataState(DATACONTROL_FETCH) & STATE_ENABLED) {
        if (DataObjects[i]->getOrder() != -1) {
          if (z)
            sQuery += ",";
          else
            sQuery += " ORDER BY ";
          sQuery += getFullName(DataObjects[i]);
          switch (DataObjects[i]->getOrder()) {
          case 1:
            sQuery += " DESC";
            break;
          case 0:
            sQuery += " ASC";
            break;
          }
          z++;
        }
        if (DataObjects[i]->DataLookup) {
          if (DataObjects[i]->DataLookup->getOrder() != -1) {
            if (z)
              sQuery += ",";
            else
              sQuery += " ORDER BY ";
            sQuery += getFullName(DataObjects[i]->DataLookup);
            switch (DataObjects[i]->DataLookup->getOrder()) {
            case 1:
              sQuery += " DESC";
              break;
            case 0:
              sQuery += " ASC";
              break;
            }
            z++;
          }
        }
      }
    }


    // Range Size & Limit
    if (iRangeSize != -1) {
      sQuery += " LIMIT ";
      // SQL Syntax Depends on the selected DatabaseType
      switch (Database->getDatabaseType()) {
      case DATABASE_POSTGRES:
        sQuery += intstr(iRangeSize);
        if (iRangeStart) sQuery += "," + intstr(iRangeStart);
        break;
      default:
        if (iRangeStart) sQuery += intstr(iRangeStart) + ",";
        sQuery += intstr(iRangeSize);
      }
    }


    APL_TRACEVAR("CAPLDBControl::fetch", "Query", sQuery);

    try {
      if (!handleDB) Database->connect();
      Result = (CAPLDBResult *) Database->openQuery(sMainHead + sQuery, bufferSize);

      // Write the result
      while (Result->fetch()) {
        int rowIndex = 1;
        string primKey = Result->get(0);
        dataset DataSet;

        for (unsigned int i = 0; i < DataObjects.size(); i++) {
          if (DataObjects[i]->getDataState(DATACONTROL_FETCH) & STATE_ENABLED) {
            string tmpValue = Result->get(rowIndex);
            DataSet[DataObjects[i]->getName()] = tmpValue;
            rowIndex++;
          }
        }
        addData(primKey, DataSet, true);
        if (hasFilter) setPrimaryValue(primKey);
      }
      Database->closeQuery(Result);
      if (!handleDB) Database->disconnect();
    } catch (apldatabase_error E) {
      throw runtime_error(E.what());
    }
  }
  APL_TRACEOUT("CAPLDBControl::fetch");
}

string CAPLDBControl::add(const string &sKey, dataset &Hash) {
  APL_TRACEIN("CAPLDBControl::add");

  check();
  bool handleDB = Database->isConnected();
  string sSQL;
  int iNewID;
  string newID;

  // Execute CAPLDataControlObject Event: EVENT_ADD_BEFORE
  iNewID = executeEvent(EVENT_ADD_BEFORE, NULL, &Hash);
  if (iNewID) return intstr(iNewID);

  if (!handleDB) Database->connect();

  // If there's no key we calculate one from the table and the primary key
  if (sKey != "") {
    newID = sKey;
  } else {
    iNewID = Database->getNewID(sTable, sPrimaryKey);
    newID = intstr(iNewID);
  }

  // Build the SQL Statement
  unsigned int fieldCount = DataObjects.size();

  sSQL = "INSERT INTO " + sTable;
  sSQL += " (" + sPrimaryKey;

  for (unsigned int i = 0; i < fieldCount; i++) {
    string name = DataObjects[i]->getName();
    if (DataObjects[i]->getDataState(DATACONTROL_ADD) & STATE_ENABLED) {
      if (sPrimaryKey != name)
        sSQL += "," + name;
      else
        newID = Hash[name];
    }
  }

  sSQL += ") VALUES ('" + newID + "'";

  for (unsigned int i = 0; i < fieldCount; i++) {
    string name = DataObjects[i]->getName();
    if (DataObjects[i]->getDataState(DATACONTROL_ADD) & STATE_ENABLED) {
      if (sPrimaryKey != name) sSQL += ",'" + Hash[name] + "'";
    }
  }

  sSQL += ")";

  // Execute the SQL Statement
  APL_TRACEVAR("CAPLDBControl::add", "Query", sSQL);
  Database->execQuery(sSQL.c_str());
  executeEvent(EVENT_ADD_AFTER, &newID, &Hash);
  if (!handleDB) Database->disconnect();

  APL_TRACEOUTRET("CAPLDBControl::add", newID);
  return newID;
}

string CAPLDBControl::modify(const string &sKey, dataset &Hash) {
  APL_TRACEIN("CAPLDBControl::modify");
  CAPLDBResult * Result;
  check();
  bool handleDB = Database->isConnected();
  string sSQL;
  int z;

  if (!handleDB) Database->connect();

  // Check wether Dataset with specified key exists in DB
  Result = (CAPLDBResult*) Database->openQuery("SELECT count(" + sPrimaryKey + ") FROM " + sTable + " WHERE " + sPrimaryKey + " = '" + sKey + "'");
  Result->fetch();
  if (Result->get(0) == "0") {
    string sReturn = add(sKey, Hash);
    return sReturn;
  }

  // Execute CAPLDataControlObject Event: EVENT_MODIFY_BEFORE
  if (executeEvent(EVENT_MODIFY_BEFORE, &sKey, &Hash)) return "";

  // Create Update Query
  sSQL = "UPDATE " + sTable + " SET ";
  z = 0;
  for (unsigned int i = 0; i < DataObjects.size(); i++) {
    if (DataObjects[i]->getDataState(DATACONTROL_MODIFY) & STATE_ENABLED) {
      if (z > 0) sSQL += ",";
      sSQL += DataObjects[i]->getName() + "='" + Hash[DataObjects[i]->getName()] + "'";
      z++;
    }
  }
  sSQL += " WHERE " + sPrimaryKey + "='" + sKey + "'";

  APL_TRACEVAR("CAPLDBControl::modify", "Query", sSQL);
  Database->execQuery(sSQL.c_str());

  // Execute CAPLDataControlObject Event: EVENT_MODIFY_AFTER
  executeEvent(EVENT_MODIFY_AFTER, &sKey, &Hash);

  if (!handleDB) Database->disconnect();

  APL_TRACEOUT("CAPLDBControl::modify");

  return sKey;
}

void CAPLDBControl::remove(const string &sKey) {
  APL_TRACEIN("CAPLDBControl::remove");

  check();
  bool handleDB = Database->isConnected();
  string sSQL;

  // Execute CAPLDataControlObject Event: EVENT_REMOVE_BEFORE
  if (executeEvent(EVENT_REMOVE_BEFORE, &sKey, NULL)) return ;

  // Create SQL Query
  sSQL = "DELETE FROM " + sTable;
  sSQL += " WHERE " + sPrimaryKey + "='" + sKey + "'";

  if (!handleDB) Database->connect();

  APL_TRACEVAR("CAPLDBControl::remove", "Query", sSQL);
  Database->execQuery(sSQL.c_str());

  // Execute CAPLDataControlObject Event: EVENT_REMOVE_AFTER
  executeEvent(EVENT_REMOVE_AFTER, &sKey, NULL);

  if (!handleDB) Database->disconnect();

  APL_TRACEOUT("CAPLDBControl::remove");
}
