/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplengine.cpp,v 1.40 2002/09/24 15:04:21 k Exp $ */


#include "apl_config.h"
#include "aplengine.h"
#include "aplcachedtemplate.h"
#include "aplexecutor.h"
#include "aplcachedstyle.h"
#include <sys/stat.h>
#include <unistd.h>

using namespace std;

static map < string, CAPLCachedTemplate * > mCache;
static map < string, CAPLCachedStyle * > cachedStyles;
static int markIdCounter;

CAPLEngine::CAPLEngine() {
  templateRoot = APLG(sTemplateRoot);
  templateType = APLG(sTemplateType);
  templateAlias = APLG(sTemplateAlias);
  theme = APLG(sTheme);
  style = APLG(sStyle);
  styleFile = APLG(styleFile);
  markId = 0;
  templateName = "template";
}

void CAPLEngine::setTheme(const string &theme) {
  this->theme = theme;
}

string CAPLEngine::getTheme(void) {
  return theme;
}

void CAPLEngine::setStyle(const string &style) {
  this->style = style;
}

string CAPLEngine::getStyle(void) {
  return style;
}

void CAPLEngine::setTemplateDir(const string &templateRoot, const string &templateAlias) {
  this->templateRoot = templateRoot;
  this->templateAlias = templateAlias;
}

string CAPLEngine::getTemplateDir(void) {
  return templateRoot;
}

void CAPLEngine::setTemplateAlias(const string &templateAlias) {
  this->templateAlias = templateAlias;
}

string CAPLEngine::getTemplateAlias(void) {
  return templateAlias;
}

void CAPLEngine::setTemplateType(const string &sNewType) {
  templateType = sNewType;
}

string CAPLEngine::getTemplateType(void) {
  return templateType;
}

void CAPLEngine::setTemplate(const string &sTemplate, const string &sAlternativeTemplate) {
  templateName = sTemplate;
  templateAlternative = sAlternativeTemplate;
}

string CAPLEngine::getTemplate(void) {
  return templateName;
}

string CAPLEngine::getAlternativeTemplate(void) {
  return templateAlternative;
}

int CAPLEngine::getMarkId(void) {
  return this->markId;
}

void CAPLEngine::setMarkId(int newMarkId) {
  this->markId = newMarkId;
}

void CAPLEngine::processStyleFile(void) {
  CAPLCachedStyle *style;
  string filename = getStyleDir(true) + "/" + styleFile;
  style = cachedStyles[filename];

  if (style) {
    if (!style->isRecent()) {
      delete style;
      style = NULL;
    }
  }

  if (!style) {
    style = new CAPLCachedStyle(filename);
    cachedStyles[filename] = style;
  }


  for (map < string, string > ::iterator i = style->arrayData.begin(); i != style->arrayData.end(); i++) {
    setStyleVar(i->first, i->second);
  }
}

void CAPLEngine::setStyleFile(const string &styleFile) {
  this->styleFile = styleFile;
}

string CAPLEngine::getStyleFile(void) {
  return styleFile;
}

void CAPLEngine::newMarkId(void) {
  setMarkId(++markIdCounter);
}

string CAPLEngine::getThemeDir(const bool &bAbs) {
  if (bAbs == true) return (templateRoot + "/" + theme);
  if (!templateAlias.length()) return getThemeDir(true);
 
  return (templateAlias + "/" + theme);
}

string CAPLEngine::getStyleDir(const bool &bAbs) {
  return getThemeDir(bAbs) + "/styles/" + style;
}

string CAPLEngine::getStyleVar(const string &key) {
  processStyleFile();
  return CAPLExecutor::getStyleVar(key);
}

string CAPLEngine::getTemplateFile(const string &sTemplateName, const string &sFilename) {
  return getThemeDir(true) + "/" + sTemplateName + "/" + templateType + "/" + sFilename + TEMPLATESUFFIX;
}

bool CAPLEngine::existsTemplateFile(const string &sTemplateFile) {
  struct stat sbuf;

  if (stat(sTemplateFile.c_str(), &sbuf) == -1)
    return false;
  else
    return true;
}

string CAPLEngine::outputTemplate(const string &sFilename) {
  string sTemplateFile = "";
  // These calls need to remain here as CAPLExecutor depends on these vars
  sStyleDir = getStyleDir(false);
  sThemeDir = getThemeDir();

  // Set the style file
  processStyleFile();

  if (!existsTemplateFile(getTemplateFile(templateName, sFilename)) && templateAlternative.length() > 0)
    sTemplateFile = getTemplateFile(templateAlternative, sFilename);
  else
    sTemplateFile = getTemplateFile(templateName, sFilename);

  // Template Caching
  CAPLCachedTemplate *cTemplate;
  cTemplate = mCache[sTemplateFile];

  if (cTemplate && !cTemplate->isRecent()) {
    delete cTemplate;
    mCache[sTemplateFile] = cTemplate = NULL;
  }

  if (!cTemplate) {
    mCache[sTemplateFile] = cTemplate = new CAPLCachedTemplate(sTemplateFile);
  }

  return cTemplate->execute(this);
}

void CAPLEngine::drawTemplate(const string &sFilename) {
  string sRes;

  sRes = outputTemplate(sFilename);
  PRINTF(sRes.c_str(), sRes.length());
}

void CAPLEngine::drawString(const string &sString) {
  PRINTF(sString.c_str(), sString.length());
}
