/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplform.cpp,v 1.31 2002/09/24 15:04:21 k Exp $ */


#include "aplform.h"
#include "aplhiddenfield.h"

using namespace strutilsxx;
using namespace std;

CAPLForm::CAPLForm(CAPLObject* Parent) : CAPLFormObject(Parent) {
  sAction = "";
  sMethod = "post";
  sEnctype = "";
  sTarget = "";
  setTemplate("form");
  ActionHandler = NULL;
}

void CAPLForm::setAction(const string &sAction) {
  this->sAction = sAction;
}

void CAPLForm::setMethod(const string &sMethod) {
  this->sMethod = sMethod;
}

void CAPLForm::setEnctype(const string &sEnctype) {
  this->sEnctype = sEnctype;
}

void CAPLForm::setTarget(const string &sTarget) {
  this->sTarget = sTarget;
}

void CAPLForm::setActionHandler(CAPLActionHandler *ActionHandler) {
  this->ActionHandler = ActionHandler;
}

string CAPLForm::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";

    prepare();
    setIntern("Method", sMethod);
    setIntern("Name", getName());
    if (!sAction.length() && ActionHandler) {
      // We set the page here if none is specified
      sAction = ActionHandler->getPage();
    }
    setIntern("Action", sAction);
    if (sEnctype != "") sParams += "enctype=\"" + sEnctype + "\" ";
    if (sTarget != "") sParams += "target=\"" + sTarget + "\" ";
    setIntern("Optional", sParams);

    if (ActionHandler) {
      // If an object of type CAPLActionHandler is passed, we pass
      // Action Vars via HiddenFields
      CAPLHiddenField HiddenField;
      map < string, string > ActionVars = ActionHandler->getVars();

      for (map < string, string > ::iterator Iterate = ActionVars.begin(); Iterate != ActionVars.end(); Iterate++) {
        HiddenField.setName((*Iterate).first);
        HiddenField.setValue((*Iterate).second);
        addChildren(HiddenField.output());
      }
    }
    output += outputTemplate();
  }
  return output;
}
