/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplformcontrolobject.cpp,v 1.11 2002/09/24 15:04:21 k Exp $ */


#include "aplformcontrolobject.h"

using namespace std;

CAPLFormControlObject::CAPLFormControlObject(CAPLObject* Parent) : CAPLObject(Parent) {
  ActionHandler = NULL;
  sError = "";
  bFailure = false;
}

void CAPLFormControlObject::setCallback(int type, apl_callback func, apl_userdat_dtor dtor, void *data) {
  if (type < 0 || type > ACTION_NUMBER) throw runtime_error("Callback specifier exceeds limit");
  event[type].setCallback(func, dtor, data);
}

void CAPLFormControlObject::setFailureCallback(int type, apl_callback func, apl_userdat_dtor dtor, void *userdat) {
  failureCallback[type].setCallback(func, dtor, userdat);
}

void CAPLFormControlObject::setActionHandler(CAPLActionHandler *ActionHandler) {
  this->ActionHandler = ActionHandler;
  ActionHandler->addActionObject(this);
}

int CAPLFormControlObject::executeEvent(int iType, const string *sKey, dataset *mHash) {
  apl_formcontrol_action action;
  string sTmp = getName();

  action.sKey = sKey;
  action.mHash = mHash;
  action.sName = &sTmp;

  return event[iType].execute(&action);
}

void CAPLFormControlObject::executeFailureCallback(const string &error, const string &key, const string &field, const string &value, dataset &hash, int mode) {
  this->setError(error);
  this->setFailure(true);
  apl_form_failure data;
  data.sKey = &key;
  data.sField = &field;
  data.sValue = &value;
  data.mHash = &hash;

  failureCallback[mode].execute(&data);

  if (mode == ACTION_ADD || mode == ACTION_MODIFY)
    failureCallback[ACTION_STANDARD].execute(&data);
}

void CAPLFormControlObject::setError(const string &sError) {
  setIntern("Error", sError);
  setIntern("Error", outputTemplate("error"));
  this->sError = sError;
}

string CAPLFormControlObject::getError(void) {
  return this->sError;
}

bool CAPLFormControlObject::getFailure(void) {
  return this->bFailure;
}

void CAPLFormControlObject::setFailure(bool bFailure) {
  this->bFailure = bFailure;
}
