/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplmap.cpp,v 1.8 2002/09/24 15:04:22 k Exp $ */


#include "aplmap.h"
#include <strutilsxx.h>

using namespace std;
using namespace strutilsxx;

CAPLMap::CAPLMap(CAPLObject* Parent) : CAPLObject(Parent) {
  setTemplate("map");
  ActionHandler = NULL;
}

CAPLMap::~CAPLMap() {};

string CAPLMap::output() {
  string sOutput;
  string sParams;

  if (getState() & STATE_VISIBLE) {
    prepare();
    if (getName().length()) sParams += " name=\"" + getName() + "\"";
    setIntern("Optional", sParams);
    sOutput = outputTemplate();
  }
  return sOutput;
}

void CAPLMap::setActionHandler(CAPLActionHandler *NewActionHandler) {
  ActionHandler = NewActionHandler;
}

CAPLMapArea::CAPLMapArea(CAPLMap *Parent): CAPLObject(Parent) {
  if (Parent) {
    setTemplateType(Parent->getTemplateType());
    ActionHandler = Parent->ActionHandler;
  }
  setTemplate("map");
}

CAPLMapArea::~CAPLMapArea() {};

void CAPLMapArea::setShape(const long iNewShape) {
  iShape = iNewShape;
}

long CAPLMapArea::getShape(void) {
  return iShape;
}

void CAPLMapArea::setAlt(const string &sNewAlt) {
  sAlt = sNewAlt;
}

string &CAPLMapArea::getAlt(void) {
  return sAlt;
}

void CAPLMapArea::setTarget(const string &sNewTarget) {
  sTarget = sNewTarget;
}

string &CAPLMapArea::getTarget(void) {
  return sTarget;
}

void CAPLMapArea::setLink(const string &sNewLink) {
  sLink = sNewLink;
}

string &CAPLMapArea::getLink(void) {
  return sLink;
}

void CAPLMapArea::setTitle(const string &sNewTitle) {
  sTitle = sNewTitle;
}

string &CAPLMapArea::getTitle(void) {
  return sTitle;
}

void CAPLMapArea::setCoords(vector < long > &vNewCoords) {
  vCoords = vNewCoords;
}

vector < long > &CAPLMapArea::getCoords() {
  return vCoords;
}

string CAPLMapArea::output() {
  string sOutput, sParams;
  unsigned int iIndex;
  string sHRef;

  if (getState() & STATE_VISIBLE) {
    prepare();
    switch (iShape) {
    case SHAPE_RECT:
      sParams += " shape=\"rect\"";
      break;
    case SHAPE_CIRCLE:
      sParams += " shape=\"circle\"";
      break;
    default:
      sParams += " shape=\"polygon\"";
    }
    if (vCoords.size()) {
      sParams += " coords=\"";
      for (iIndex = 0; iIndex < vCoords.size(); iIndex++) {
        if (iIndex) sParams += ",";
        sParams += intstr(vCoords[iIndex]);
      }
      sParams += "\"";
    }
    if (sLink.length() || ActionHandler) {
      if (sLink.length())
        sHRef = getQueryString(sLink);
      else
        sHRef = getQueryString(ActionHandler->getPage());
      sParams += " href=\"" + sHRef + "\"";
    }
    if (sTarget.length()) sParams += " target=\"" + sTarget + "\"";
    if (sTitle.length()) sParams += " title=\"" + sTitle + "\"";
    sParams += " alt=\"" + sAlt + "\"";
    setIntern("Optional", sParams);
    sOutput = outputTemplate("area");
  }
  return sOutput;
}
