/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplobject.cpp,v 1.28 2002/09/24 15:04:22 k Exp $ */


#include "aplobject.h"
#include <stdexcept>
#include "strutilsxx.h"

using namespace std;
using namespace strutilsxx;

CAPLObject::CAPLObject(CAPLObject* Parent) {
  state = 0;
  setState(STATE_VISIBLE);
  setState(STATE_READWRITE);
  setState(STATE_ENABLED);
  setValue("");
  setName(getUniqueName());
  
  if (Parent) {
    this->Parent = Parent;
    Parent->addChild(this);
    setMarkId(Parent->getMarkId());
  } else {
    newMarkId();
    Parent = NULL;
  }
}

void CAPLObject::addChild(CAPLObject* Child) {
  Children.push_back(Child);
}

void CAPLObject::addSlot(const string &key, CAPLObject* Slot) {
  Slots[key] = Slot;
}

int CAPLObject::getChildrenCount(void) {
  return Children.size();
}

CAPLObject* CAPLObject::getChild(const unsigned int iIndex) {
  if (iIndex < 0 || iIndex >= Children.size())
    throw runtime_error("Cannot access non-existant child object");
  return Children[iIndex];
}

CAPLObject* CAPLObject::getParent(void) {
  return Parent;
}

void CAPLObject::setState(const int state) {
  if (state & 1) {
    this->state &= (~state);
  } else {
    this->state |= state;
  }
}

int CAPLObject::getState(void) {
  return state;
}

void CAPLObject::prepareSlots(void) {
  for (map < string, CAPLObject* > ::iterator Iterate = Slots.begin(); Iterate != Slots.end(); Iterate++) {
    if ((*Iterate).second->getMarkId() != getMarkId()) {
      (*Iterate).second->setMarkId(getMarkId());
      setSlot((*Iterate).first, (*Iterate).second->output());
    }
  }
}

void CAPLObject::prepareChildren(void) {
  setChildren(CAPLObject::output());
}

void CAPLObject::prepare(void) {
  prepareSlots();
  prepareChildren();
}

void CAPLObject::draw() {
  drawString(output());
}

string CAPLObject::output() {
  string output = "";
  for (unsigned int i = 0; i < Children.size(); i++) output += Children[i]->output();
  return output;
}

void CAPLObject::setValue(const string &value) {
  this->sValue = value;
}

string CAPLObject::getValue(void) {
  return sValue;
}

void CAPLObject::setName(const string &sName) {
  this->sName = sName;
}

string CAPLObject::getName(void) {
  return sName;
}

string CAPLObject::getUniqueName(void) {
  return "apl" + intstr(++(APLG(iUniqueName)));
}

void CAPLObject::initUniqueName(void) {
  APLG(iUniqueName) = 0;
}
