/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aploplist.cpp,v 1.23 2002/09/24 15:04:22 k Exp $ */


#include <fstream>
#include "aploplist.h"
#include "aplops.h"
#include "aplscanner.h"
#include "php_apl_public.h"


using namespace std;

CAPLOpList::CAPLOpList(const string &filename) {
  ifstream input;

  last_op = OP_INVALID;
  input.open(filename.c_str());
  apl_scan_template(this, &input);
}

string CAPLOpList::execute(CAPLExecutor *executor) {
  string result("");

  for (vector < CAPLOp* > ::iterator iterate = ops.begin();
       iterate != ops.end();
       iterate++) {
    result.append((*iterate)->execute(executor));
  }
  return result;
}

void CAPLOpList::handle_op(int op, const char *arg1, size_t arg1_len) {
  switch (op) {
  case OP_VAR:
    ops.push_back(new CAPLOpVar(arg1, arg1_len));
    break;
  case OP_SLOT:
    ops.push_back(new CAPLOpSlot(arg1, arg1_len));
    break;
  case OP_INTERN:
    ops.push_back(new CAPLOpIntern(arg1, arg1_len));
    break;
  case OP_GLOBAL:
    ops.push_back(new CAPLOpGlobal(arg1, arg1_len));
    break;
  case OP_CHILDREN:
    ops.push_back(new CAPLOpChildren);
    break;
  case OP_THEMEDIR:
    ops.push_back(new CAPLOpThemeDir);
    break;
  case OP_THEME:
    ops.push_back(new CAPLOpTheme);
    break;
  case OP_STYLEDIR:
    ops.push_back(new CAPLOpStyleDir);
    break;
  case OP_STYLE:
    ops.push_back(new CAPLOpStyle(arg1, arg1_len));
    break;
  case OP_NEWLINE:
    ops.push_back(new CAPLOpNewline(arg1_len));
    break;
  case OP_STRING:
    if (last_op == OP_STRING) {
      // Merge strings
      static_cast < CAPLOpString * > (ops.back())->append(arg1, arg1_len);
    } else {
      ops.push_back(new CAPLOpString(arg1, arg1_len));
    }
    break;
  }
  last_op = op;
}
