/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: aplquerystring.cpp,v 1.4 2002/09/24 15:04:22 k Exp $ */

#include "aplquerystring.h"

using namespace std;

CAPLQueryString::CAPLQueryString(void) { 
  ActionHandler = NULL;
};

CAPLQueryString::~CAPLQueryString(void) {
};

void CAPLQueryString::clearQueryVars(void) {
  mQueryVars.erase(mQueryVars.begin(), mQueryVars.end());
}

bool CAPLQueryString::existsQueryVar(const string &sKey) {
  if (mQueryVars.count(sKey))
    return true;
  else
    return false;
}

void CAPLQueryString::setQueryVar(const string &sKey, const string &sValue, const bool &bOverwrite) {
  if (!existsQueryVar(sKey) || (existsQueryVar(sKey) && bOverwrite)) {
    mQueryVars[sKey] = sValue;
  }
}

string CAPLQueryString::getQueryVar(const string &sKey) {
  if (mQueryVars.count(sKey))
    return mQueryVars[sKey];
  else
    return "";
}

void CAPLQueryString::delQueryVar(const string &sKey) {
  mQueryVars.erase(sKey);
}

string CAPLQueryString::getQueryString(const string &sLink) {
  string sQueryString;
  bool bFirst;
  map< string, string > mActionVars;
  map< string, string >::iterator IPos;

  sQueryString = sLink;
  bFirst = sQueryString.find("?") == string::npos;
  if (ActionHandler) {
    mActionVars = ActionHandler->getVars();
    IPos = mActionVars.begin();
    while (IPos != mActionVars.end()) {
      setQueryVar((*IPos).first, (*IPos).second, false);
      IPos++;
    }
  }
  IPos = mQueryVars.begin();
  while (IPos != mQueryVars.end()) {
    if (bFirst) {
      sQueryString += "?";
      bFirst = false;
    }
    else
      sQueryString += "&amp;";
    sQueryString += (*IPos).first + "=" + (*IPos).second;
    IPos++;
  }
  return sQueryString;
}

void CAPLQueryString::setActionHandler(CAPLActionHandler *NewActionHandler) {
  ActionHandler = NewActionHandler;
}
