%{
/*
   +----------------------------------------------------------------------+
   | APL - Abstract Presentation Layer                                    |
   +----------------------------------------------------------------------+
   | Copyright (c) 2000 Andre Christ <a.christ@twisd.de>, twisd AG        |
   +----------------------------------------------------------------------+
   | This library is free software; you can redistribute it and/or        |
   | modify it under the terms of the GNU Library General Public          |
   | License as published by the Free Software Foundation; either         |
   | version 2 of the License, or (at your option) any later version.     |
   |                                                                      |
   | This library is distributed in the hope that it will be useful,      |
   | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
   | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
   | Library General Public License for more details.                     |
   |                                                                      |
   | You should have received a copy of the GNU Library General Public    |
   | License along with this progam; if not, write to the                 |
   | Free Software Foundation, Inc., 59 Temple Place - Suite 330,         |
   | Boston, MA 02111-1307, USA.                                          |
   +----------------------------------------------------------------------+
*/

using namespace std;

// To pass a state object to the lexer function, we need
// to rename the yylex function, because we cannot change the
// respective declaration in the class definition.

#define YY_DECL int CAPLFlexLexer::yylex_ext(apl_scanner_state *apl)

#include "aploplist.h"

typedef struct {
  CAPLOpList *op_list;
  int current_op;
} apl_scanner_state;

#include <stdexcept>
  
// aplFlexLexer is flex's class. We add the new name for yylex to
// that class by embracing the original class.
  
class CAPLFlexLexer : public aplFlexLexer 
{
public:
  int yylex_ext(apl_scanner_state *);
};

void apl_scan_template(CAPLOpList *op_list, istream *in)
{
  CAPLFlexLexer lexer;
  apl_scanner_state state;

  state.op_list = op_list;
  state.current_op = OP_INVALID;
  
  lexer.switch_streams(in, &cerr);
  while (lexer.yylex_ext(&state) != 0);
}

int aplFlexLexer::yylex(void)
{
  throw std::runtime_error("The original yylex must not be called");
  return 0;
}

%}

ID              [a-z][a-z0-9]*
ALMOST_ANYTHING ([^$\n])+
NEWLINE         \n
ANY             (.|\n)

%s ST_VAR

%option noyywrap
%option prefix="apl"

%%

<INITIAL>"$VAR" {
  apl->current_op = OP_VAR; 
  BEGIN(ST_VAR);
}

<INITIAL>"$SLOT" {
  apl->current_op = OP_SLOT;
  BEGIN(ST_VAR);
}

<INITIAL>"$INTERN" {
  apl->current_op = OP_INTERN;
  BEGIN(ST_VAR);
}

<INITIAL>"$GLOBAL" {
  apl->current_op = OP_GLOBAL;
  BEGIN(ST_VAR);
}

<INITIAL>"$STYLE" {
  apl->current_op = OP_STYLE;
  BEGIN(ST_VAR);
}

<INITIAL>"$CHILDREN"     apl->op_list->handle_op(OP_CHILDREN);
<INITIAL>"$STYLEDIR"     apl->op_list->handle_op(OP_STYLEDIR);
<INITIAL>"$THEMEDIR"     apl->op_list->handle_op(OP_THEMEDIR);
<INITIAL>"$THEME"        apl->op_list->handle_op(OP_THEME);
<INITIAL>{NEWLINE} {
  apl->op_list->handle_op(OP_NEWLINE, YYText(), YYLeng());
}

<INITIAL>{ALMOST_ANYTHING} {
  apl->op_list->handle_op(OP_STRING, YYText(), YYLeng());
}

<INITIAL>"$" {
  apl->op_list->handle_op(OP_STRING, YYText(), 1);
}

<ST_VAR>"["{ID}"]" {
  apl->op_list->handle_op(apl->current_op, YYText() + 1, YYLeng() - 2); 
  BEGIN(INITIAL);
}

<ST_VAR>{ANY} {
  // We could throw a runtime error here or output a
  // diagnostic message.
  unput(YYText()[0]);
  BEGIN(INITIAL);
}


%%