%{
/*
   +----------------------------------------------------------------------+
   | APL - Abstract Presentation Layer                                    |
   +----------------------------------------------------------------------+
   | Copyright (c) 2000 Andre Christ <a.christ@twisd.de>, twisd AG        |
   +----------------------------------------------------------------------+
   | This library is free software; you can redistribute it and/or        |
   | modify it under the terms of the GNU Library General Public          |
   | License as published by the Free Software Foundation; either         |
   | version 2 of the License, or (at your option) any later version.     |
   |                                                                      |
   | This library is distributed in the hope that it will be useful,      |
   | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
   | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
   | Library General Public License for more details.                     |
   |                                                                      |
   | You should have received a copy of the GNU Library General Public    |
   | License along with this progam; if not, write to the                 |
   | Free Software Foundation, Inc., 59 Temple Place - Suite 330,         |
   | Boston, MA 02111-1307, USA.                                          |
   +----------------------------------------------------------------------+
*/

using namespace std;


// To pass a state object to the lexer function, we need
// to rename the yylex function, because we cannot change the
// respective declaration in the class definition.

#define YY_DECL int CAPLSDFFlexLexer::yylex_ext(apl_sdf_state *apl)

#include <map>
#include <string>
  
typedef struct {
  string variable_name;
  map<string,string> *data;
} apl_sdf_state;

#include <stdexcept>
  
// aplFlexLexer is flex's class. We add the new name for yylex to
// that class by embracing the original class.
  
class CAPLSDFFlexLexer : public aplsdfFlexLexer 
{
public:
  int yylex_ext(apl_sdf_state *);
};

void aplScanSDF(istream *in, map<string,string> &data)
{
  CAPLSDFFlexLexer lexer;
  apl_sdf_state state;

  state.data = &data;

  lexer.switch_streams(in, &cerr);
  while (lexer.yylex_ext(&state) != 0);
}

int aplsdfFlexLexer::yylex(void)
{
  throw runtime_error("The original yylex must not be called");
  return 0;
}

%}

ANYTHING        .*
NAME            [a-zA-Z0-9_]+
NEWLINE         \n
WS              [ \t\v]+
NON_WS          [^ \t\v]

%s ST_COLON
%s ST_VAR_CONTENT

%option noyywrap
%option prefix="aplsdf"

%%

<INITIAL>{NAME} {
  apl->variable_name.assign(YYText(), YYLeng());
  BEGIN(ST_COLON);
}

<ST_COLON>":" {
  BEGIN(ST_VAR_CONTENT);
}

<ST_VAR_CONTENT>{NON_WS}{ANYTHING} {
  (*apl->data)[apl->variable_name].assign(YYText(), YYLeng());
  BEGIN(INITIAL);
}

<INITIAL,ST_COLON,ST_VAR_CONTENT>{WS}   { }
<INITIAL>"#"{ANYTHING}                  { }
<INITIAL>"\n"                           { }
<INITIAL,ST_COLON,ST_VAR_CONTENT>.      { }

%%
