/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apltabfolder.cpp,v 1.39 2002/09/24 15:04:22 k Exp $ */


#include "apltabfolder.h"
#include "apl_config.h"
#include "aplquerystring.h"

using namespace strutilsxx;
using namespace std;

CAPLTabFolderPanel::CAPLTabFolderPanel(CAPLObject* Parent) : CAPLObject(Parent) {
  sWidth = "300";
  sHeight = "300";
  sCaption = "";
  oldTab = "";
  ActionHandler = NULL;
  setTemplate("tabfolderpanel");
}

void CAPLTabFolderPanel::addTabFolder(CAPLTabFolder* TabFolder) {
  TabFolders.push_back(TabFolder);
  TabFolder->setName(intstr(TabFolders.size()));
}

void CAPLTabFolderPanel::setWidth(const string &sNewWidth) {
  sWidth = sNewWidth;
}

string CAPLTabFolderPanel::getWidth(void) {
  return sWidth;
}

void CAPLTabFolderPanel::setHeight(const string &sNewHeight) {
  sHeight = sNewHeight;
}

string CAPLTabFolderPanel::getHeight(void) {
  return sHeight;
}

void CAPLTabFolderPanel::setCaption(const string &sNewCaption) {
  sCaption = sNewCaption;
}

void CAPLTabFolderPanel::setActionHandler(CAPLActionHandler* ActionHandler) {
  this->ActionHandler = ActionHandler;
  ActionHandler->addActionObject(this);
}

bool CAPLTabFolderPanel::isTabFolderPanel(void) {
  if (ActionHandler->getData("APLtf").getKey() == getName())
    return true;
  else
    return false;
}

void CAPLTabFolderPanel::setActive(const bool &bActive) {
  for (unsigned int i = 0; i < TabFolders.size(); i++) {
    TabFolders[i]->setActive(bActive);
  }
}

void CAPLTabFolderPanel::execute(void) {
  APL_TRACEIN("CAPLTabFolderPanel::execute");

  if (ActionHandler != NULL) {
    if (isTabFolderPanel()) {
      string sActiveTab = ActionHandler->data["APLtf"].data[getName()].data["Act"].value;
      oldTab = ActionHandler->data["APLtf"].data[getName()].data["Old"].value;

      for (unsigned int i = 0; i < TabFolders.size(); i++) {
        string sTabName = TabFolders[i]->getName();
        if (sActiveTab == sTabName) {
          TabFolders[i]->setActive();
          ActionHandler->setVar("APLtf[" + getName() + "][Act]", sActiveTab);
          
          // Fire callbacks
          executeEvent(ACTION_ACTIVE, &sTabName);
          if (oldTab != "") executeEvent(ACTION_ACTIVATE, &sTabName);
          if (oldTab != "" && sActiveTab != oldTab) executeEvent(ACTION_CHANGE, &sTabName);
          
          oldTab = sTabName;
        }
      }
    }
  }

  APL_TRACEOUT("CAPLTabFolderPanel::execute");
}

string CAPLTabFolderPanel::output(void) {
  APL_TRACEIN("CAPLTabFolderPanel::output");
  string output = "";
  if (getState() & STATE_VISIBLE) {

    for (unsigned int i = 0; i < TabFolders.size(); i++) {
      output += TabFolders[i]->outputCaption();
      if (i < (TabFolders.size() - 1)) output += outputTemplate("separator");
    }
    setIntern("Captions", output);
    output = "";
    for (unsigned int i = 0; i < TabFolders.size(); i++)
      output += TabFolders[i]->outputBody();
    prepareSlots();
    setChildren(output);

    setIntern("Width", sWidth);
    setIntern("Height", sHeight);
    setIntern("Caption", sCaption);
    output = outputTemplate();
  }
  APL_TRACEOUT("CAPLTabFolderPanel::output");
  return output;
}

void CAPLTabFolderPanel::setCallback(int type, apl_callback func, apl_userdat_dtor dtor, void *data) {
  if (type < 0 || type > ACTION_NUMBER)
    throw runtime_error("Callback specifier exceeds limit");
  event[type].setCallback(func, dtor, data);
}

int CAPLTabFolderPanel::executeEvent(int type, const string *key) {
  apl_tabfolder_action action;

  action.key = key;
  return event[type].execute(&action);
}

CAPLTabFolder::CAPLTabFolder(CAPLTabFolderPanel* Parent) {
  Parent->addTabFolder(this);
  TabFolderPanel = Parent;
  
  setTemplate("tabfolderpanel");
  sCaption = "";
  sLink = "";
  bActive = false;  
}

void CAPLTabFolder::setCaption(const string &sNewCaption) {
  sCaption = sNewCaption;
}

void CAPLTabFolder::setLink(const string &sLink) {
  this->sLink = sLink;
}

void CAPLTabFolder::setActive(const bool &bActive) {
  if (bActive) TabFolderPanel->setActive(!bActive);
  this->bActive = bActive;
}

bool CAPLTabFolder::isActive(void) {
  return bActive;
}

string CAPLTabFolder::outputCaption() {
  APL_TRACEIN("CAPLTabFolder::outputCaption");
  string output = "";
  CAPLQueryString QueryString;

  if (getState() & STATE_VISIBLE) {
    string sHREF;

    if (sLink.length() || TabFolderPanel->ActionHandler) {
      QueryString.setActionHandler(TabFolderPanel->ActionHandler);

      if (sLink.length()) {
        sHREF = QueryString.getQueryString(sLink);
      } else {
        QueryString.setQueryVar("APLtf[" + TabFolderPanel->getName() + "][Act]", getName());
        string sOldTab = TabFolderPanel->oldTab;
        if (!sOldTab.length()) sOldTab = getName();
        QueryString.setQueryVar("APLtf[" + TabFolderPanel->getName() + "][Old]", sOldTab); 
        sHREF = QueryString.getQueryString(TabFolderPanel->ActionHandler->getPage());
      }
      setIntern("Link", sHREF);
    }

    // Caption 
    setIntern("Caption", sCaption);
    if (bActive)
      output += outputTemplate("tabfolder/captionactive");
    else
      output += outputTemplate("tabfolder/caption");
  }

  APL_TRACEOUT("CAPLTabFolder::outputCaption");

  return output;
}

string CAPLTabFolder::outputBody() {
  APL_TRACEIN("CAPLTabFolder::outputBody");

  string output = "";
  if (getState() & STATE_VISIBLE) {
    if (bActive) {
      setIntern("Width", TabFolderPanel->getWidth());
      setIntern("Height", TabFolderPanel->getHeight());
      prepare();
      output += outputTemplate("tabfolder/main");
    }
  }

  APL_TRACEOUT("CAPLTabFolder::outputBody");

  return output;
}
