/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apltable.cpp,v 1.35 2002/09/24 15:04:22 k Exp $ */


#include "apltable.h"

using namespace strutilsxx;
using namespace std;

CAPLTable::CAPLTable(CAPLObject* Parent) : CAPLObject(Parent) {
  sWidth = "";
  sHeight = "";
  sBackground = "";
  sBgColor = "";
  iCellPadding = -1;
  iCellSpacing = -1;
  iBorder = -1;
  align = "";
  compressEmptyCells = false;
  setTemplate("table");
}

void CAPLTable::setAlign(const string &align) {
  this->align = align;
}

void CAPLTable::setWidth(const string &sNewWidth) {
  sWidth = sNewWidth;
}

void CAPLTable::setHeight(const string &sNewHeight) {
  sHeight = sNewHeight;
}

void CAPLTable::setBackground(const string &sNewBackground) {
  sBackground = sNewBackground;
}

void CAPLTable::setBgColor(const string &sNewBgColor) {
  sBgColor = sNewBgColor;
}

void CAPLTable::setCellPadding(const int &iNewCellPadding) {
  iCellPadding = iNewCellPadding;
}

void CAPLTable::setCompressEmptyCells(const bool newCompressEmptyCells) {
  compressEmptyCells = newCompressEmptyCells;
}

bool CAPLTable::getCompressEmptyCells() {
  return compressEmptyCells;
}

void CAPLTable::setCellSpacing(const int &iNewCellSpacing) {
  iCellSpacing = iNewCellSpacing;
}

void CAPLTable::setBorder(const int &iNewBorder) {
  iBorder = iNewBorder;
}

void CAPLTable::prepareDraw(void) {
  CAPLTableRow *Row, *tmpRow;
  CAPLTableCell *Cell, *tmpCell;

  for (int iRow = 0; iRow < getChildrenCount(); iRow++) {
    Row = (CAPLTableRow *) getChild(iRow);

    for (int iCol = 0; iCol < Row->getChildrenCount(); iCol++) {
      Cell = (CAPLTableCell *) Row->getChild(iCol);
      if (Cell->iColspan) {
        for (int iHiddenCol = 1; iHiddenCol < Cell->iColspan; iHiddenCol++) {
          tmpRow = (CAPLTableRow *) getChild(iRow);
          tmpCell = (CAPLTableCell *) tmpRow->getChild(iCol + iHiddenCol);
          tmpCell->bHidden = true;
          if (Cell->iRowspan) {
            for (int iHiddenRow = 1; iHiddenRow < Cell->iRowspan; iHiddenRow++) {
              tmpRow = (CAPLTableRow *) getChild(iRow + iHiddenRow);
              tmpCell = (CAPLTableCell *) tmpRow->getChild(iCol);
              tmpCell->iColspan = Cell->iColspan;
            }
          }
        }
      }

      if (Cell->iRowspan) {
        for (int iHiddenRow = 1; iHiddenRow < Cell->iRowspan; iHiddenRow++) {
          tmpRow = (CAPLTableRow *) getChild(iRow + iHiddenRow);
          tmpCell = (CAPLTableCell *) tmpRow->getChild(iCol);
          tmpCell->bHidden = true;
        }
      }
    }
  }
}

string CAPLTable::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";

    prepareDraw();
    prepare();

    if (align != "") sParams += "align=\"" + align + "\" ";
    if (sWidth != "") sParams += "width=\"" + sWidth + "\" ";
    if (sHeight != "") sParams += "height=\"" + sHeight + "\" ";
    if (sBackground != "") sParams += "background=\"" + sBackground + "\" ";
    if (sBgColor != "") sParams += "bgcolor=\"" + sBgColor + "\" ";
    if (iCellPadding != -1) sParams += "cellpadding=\"" + intstr(iCellPadding) + "\" ";
    if (iCellSpacing != -1) sParams += "cellspacing=\"" + intstr(iCellSpacing) + "\" ";
    if (iBorder != -1) sParams += "border=\"" + intstr(iBorder) + "\"";

    setIntern("Optional", sParams);
    output += outputTemplate();
  }
  return output;
}


CAPLTableRow::CAPLTableRow(CAPLObject* Parent) : CAPLObject(Parent) {
  setTemplate("table");
}

void CAPLTableRow::setBgColor(const string &sNewBgColor) {
  sBgColor = sNewBgColor;
}

void CAPLTableRow::setAlign(const string &sNewAlign) {
  sAlign = sNewAlign;
}

void CAPLTableRow::setVAlign(const string &sNewVAlign) {
  sVAlign = sNewVAlign;
}

string CAPLTableRow::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";
    CAPLTable *table;

    if (sBgColor != "") sParams += "bgcolor=\"" + sBgColor + "\" ";
    if (sAlign != "") sParams += "align=\"" + sAlign + "\" ";
    if (sVAlign != "") sParams += "valign=\"" + sVAlign + "\" ";

    setIntern("Optional", sParams);

    table = (CAPLTable *) getParent();
    if (table->getCompressEmptyCells()) {
      int nrEmpty = 0;

      for (int i = 0; i < getChildrenCount(); i++) {
        if (getChild(i)->getChildrenCount() < 1) {
          nrEmpty++;
          continue;
        } else if (nrEmpty > 0) {
          setIntern("Colspan", intstr(nrEmpty));
          output += outputTemplate("cell/compressed");
          nrEmpty = 0;
        }
        output += getChild(i)->output();
      }
      if (nrEmpty > 0) {
        setIntern("Colspan", intstr(nrEmpty));
        output += outputTemplate("cell/compressed");
      }
      setChildren(output);
    } else {
      prepareChildren();
    }
    prepareSlots();
    output = outputTemplate("row/main");
  }
  return output;
}


CAPLTableCell::CAPLTableCell(CAPLObject* Parent) : CAPLObject(Parent) {
  bHidden = false;
  iColspan = 0;
  iRowspan = 0;
  sBgColor = "";
  sBackground = "";
  sAlign = "";
  sVAlign = "";
  sWidth = "";
  sHeight = "";
  setTemplate("table");
}

void CAPLTableCell::setBgColor(const string &sNewBgColor) {
  sBgColor = sNewBgColor;
}

void CAPLTableCell::setBackground(const string &sNewBackground) {
  sBackground = sNewBackground;
}

void CAPLTableCell::setAlign(const string &sNewAlign) {
  sAlign = sNewAlign;
}

void CAPLTableCell::setVAlign(const string &sNewVAlign) {
  sVAlign = sNewVAlign;
}

void CAPLTableCell::setWidth(const string &sNewWidth) {
  sWidth = sNewWidth;
}

void CAPLTableCell::setHeight(const string &sNewHeight) {
  sHeight = sNewHeight;
}

void CAPLTableCell::setColspan(const int iNewColspan) {
  iColspan = iNewColspan;
}

void CAPLTableCell::setRowspan(const int iNewRowspan) {
  iRowspan = iNewRowspan;
}

string CAPLTableCell::output() {
  string output = "";
  if (getState() & STATE_VISIBLE) {
    string sParams = "";

    if (!bHidden) {
      if (sBgColor != "") sParams += "bgcolor=\"" + sBgColor + "\" ";
      if (sBackground != "") sParams += "background=\"" + sBackground + "\" ";
      if (sAlign != "") sParams += "align=\"" + sAlign + "\" ";
      if (sVAlign != "") sParams += "valign=\"" + sVAlign + "\" ";
      if (sWidth != "") sParams += "width=\"" + sWidth + "\" ";
      if (sHeight != "") sParams += "height=\"" + sHeight + "\" ";
      if (iColspan) sParams += "colspan=\"" + intstr(iColspan) + "\" ";
      if (iRowspan) sParams += "rowspan=\"" + intstr(iRowspan) + "\" ";

      setIntern("Optional", sParams);

      if (getChildrenCount() < 1)
        output += outputTemplate("cell/empty");
      else {
        prepare();
        output += outputTemplate("cell/main");
      }
    }
  }
  return output;
}
