/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apltext.cpp,v 1.33 2002/09/24 15:04:22 k Exp $ */


#include "apltext.h"

using namespace strutilsxx;
using namespace std;

CAPLText::CAPLText(CAPLObject* Parent) : CAPLFormObject(Parent) {
  setTemplate("text");
  sCaption = "";
  sText = "";
}

void CAPLText::setText(const string &sText) {
  this->sText = sText;
}

void CAPLText::setCaption(const string &sCaption) {
  this->sCaption = sCaption;
}

string CAPLText::output() {
  string output = "";

  if (getState() & STATE_VISIBLE) {
    if (getValue() != "") setText(getValue());
    setIntern("Text", sText);
    if (sCaption != "") {
      setIntern("Caption", sCaption);
      setIntern("Caption", outputTemplate("caption"));
    }
    output = outputTemplate();
  }

  return output;
}

string APLBold(const string &sNewText) {
  CAPLEngine Engine;

  Engine.setTemplate("text");
  Engine.setIntern("Text", sNewText);
  return Engine.outputTemplate("bold");
}

string APLUnderline(const string &sNewText) {
  CAPLEngine Engine;

  Engine.setTemplate("text");
  Engine.setIntern("Text", sNewText);
  return Engine.outputTemplate("underline");
}

string APLStrike(const string &sNewText) {
  CAPLEngine Engine;

  Engine.setTemplate("text");
  Engine.setIntern("Text", sNewText);
  return Engine.outputTemplate("strike");
}

string APLItalic(const string &sNewText) {
  CAPLEngine Engine;

  Engine.setTemplate("text");
  Engine.setIntern("Text", sNewText);
  return Engine.outputTemplate("italic");
}

string APLHeadline(const string &sNewText, const int iNewLevel) {
  CAPLEngine Engine;

  Engine.setTemplate("text");
  Engine.setIntern("Level", intstr(iNewLevel));
  Engine.setIntern("Text", sNewText);
  return Engine.outputTemplate("headline");
}

string APLHyperLink(const string &sNewLink, const string &sNewText, const string &sNewTarget) {
  CAPLEngine Engine;
  string sParams = "";


  Engine.setTemplate("text");
  Engine.setIntern("Link", sNewLink);
  if (sNewText == "")
    Engine.setIntern("Text", sNewLink);
  else
    Engine.setIntern("Text", sNewText);
  if (sNewTarget != "") sParams += "target=\"" + sNewTarget + "\" ";
  Engine.setIntern("Optional", sParams);
  return Engine.outputTemplate("hyperlink");
}

string APLBreak(const string &clear) {
  CAPLEngine Engine;

  if (clear != "") Engine.setIntern("Optional", "clear=\"" + clear + "\"");
  Engine.setTemplate("text");
  return Engine.outputTemplate("break");
}

string APLWordBreak(void) {
  CAPLEngine Engine;

  Engine.setTemplate("text");
  return Engine.outputTemplate("wordbreak");
}

string APLNoBreak(const string &text) {
  CAPLEngine Engine;

  Engine.setTemplate("text");
  Engine.setIntern("Text", text);
  return Engine.outputTemplate("nobreak");
}
