/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: apltreemenu.cpp,v 1.30 2002/09/24 15:04:22 k Exp $ */


#include "apltreemenu.h"

using namespace std;

CAPLTreeMenuItem::CAPLTreeMenuItem(CAPLTreeMenuItem *Parent) {
  if (Parent) {
    Parent->addItem(this);
    this->Parent = Parent;
  } else {
    this->Parent = NULL;
  }

  sText = "";
  sLink = "";
  sImage = "";
  sTarget = "";
  bActive = false;
  setTemplate("treemenu");
}

void CAPLTreeMenuItem::addItem(CAPLTreeMenuItem *Item) {
  Items.push_back(Item);
}

void CAPLTreeMenuItem::setText(const string &sNewText) {
  sText = sNewText;
}

void CAPLTreeMenuItem::setLink(const string &sNewLink) {
  sLink = sNewLink;
}

void CAPLTreeMenuItem::setImage(const string &sNewImage) {
  sImage = sNewImage;
}

void CAPLTreeMenuItem::setTarget(const string &sNewTarget) {
  sTarget = sNewTarget;
}

string CAPLTreeMenuItem::getTarget(void) {
  return sTarget;
}

void CAPLTreeMenuItem::setActive(const bool bNewActive, const bool onlyParents) {
  if (!onlyParents) bActive = bNewActive;
  if (Parent) Parent->setActive(bNewActive);
}

string CAPLTreeMenuItem::outputItems(const int iLevel, CAPLActionHandler *ActionHandler, CAPLTreeMenu *TopItem, const string &sGlobalTarget) {
  CAPLQueryString QueryString;
  string type = iLevel ? "level" : "toplevel";
  string output = "";

  for (unsigned int i = 0; i < Items.size(); i++) {
    if (Items[i]->getState() & STATE_VISIBLE) {
      string sIndent = "";
      string sTree = "";

      for (int z = 0; z < iLevel; z++) {
        sIndent += Items[i]->outputTemplate(type + "/indention");
      }

      if (Items[i]->Items.size() > 0) {
        if (Items[i]->bActive)
          sTree += Items[i]->outputTemplate(type + "/status/opened");
        else
          sTree += Items[i]->outputTemplate(type + "/status/closed");
      } else {
        sTree += Items[i]->outputTemplate(type + "/status/default");
      }

      Items[i]->setIntern("Indention", sIndent);

      string sOptional = "";
      Items[i]->setIntern("Status", sTree);
      Items[i]->setIntern("Text", Items[i]->sText);

      if (ActionHandler || Items[i]->sLink.length()) {
        string sHREF;

        if (ActionHandler) {
          QueryString.clearQueryVars();
          // Append the actionvars to the query
          map < string, string > ActionVars = ActionHandler->getVars();
          for (map < string, string > ::iterator Iterate = ActionVars.begin(); Iterate != ActionVars.end(); Iterate++) {
            if ((*Iterate).first.substr(0,5) != "APLtf") {
              QueryString.setQueryVar((*Iterate).first, (*Iterate).second);
            }
          }
          
          // Set Query Vars
          string sActiveItem = Items[i]->getName();
          if (Items[i]->bActive && Items[i]->Items.size())
            sActiveItem = getName();
          
          QueryString.setQueryVar("APLtm[" + TopItem->getName() + "][Act]", sActiveItem);

           // Append the old Item to query
          string old = "";
          if (Items[i]->getName() == TopItem->item) {
            old = Items[i]->getName();
          } else {
            if (TopItem->oldItem != "")
              old = TopItem->oldItem;
            else
              old = Items[i]->getName();
          }
          QueryString.setQueryVar("APLtm[" + TopItem->getName() + "][Old]", old);
        }

        if (Items[i]->sLink.length()) {
          if (Items[i]->sTarget == "") Items[i]->sTarget = sGlobalTarget;
          if (Items[i]->sTarget != "") sOptional += "target=\"" + Items[i]->sTarget + "\" ";
          sHREF = QueryString.getQueryString(Items[i]->sLink);
        } else {
          sHREF = QueryString.getQueryString(ActionHandler->getPage());
        }
        Items[i]->setIntern("Link", sHREF);
      }

      Items[i]->setIntern("Optional", sOptional);
      Items[i]->setIntern("Image", Items[i]->sImage);

      if (Items[i]->bActive)
        output += Items[i]->outputTemplate(type + "/itemactive");
      else
        output += Items[i]->outputTemplate(type + "/item");
      if (Items[i]->bActive) output += Items[i]->outputItems(iLevel + 1, ActionHandler, TopItem, sGlobalTarget);
    }
  }
  if (Items.size()) {
    setChildren(output);
    output = outputTemplate(type + "/main");
  }
  return output;
}


CAPLTreeMenu::CAPLTreeMenu(CAPLObject* Parent) {
  if (Parent) Parent->addChild(this);
  sTitle = "";
  oldItem = "";
  item = "";
  ActionHandler = NULL;
  setTemplate("treemenu");
}

void CAPLTreeMenu::setCaption(const string &sNewTitle) {
  sTitle = sNewTitle;
}

void CAPLTreeMenu::setActionHandler(CAPLActionHandler *ActionHandler) {
  this->ActionHandler = ActionHandler;
  ActionHandler->addActionObject(this);
}

void CAPLTreeMenu::setInactive(vector < CAPLTreeMenuItem* > Itms) {
  for (unsigned int i = 0; i < Itms.size(); i++) {
    Itms[i]->setActive(false);
    if (Itms[i]->Items.size()) setInactive(Itms[i]->Items);
  }
}

void CAPLTreeMenu::setActionActive(vector < CAPLTreeMenuItem* > Itms, const string &name, const bool &onlyParents = false) {
  for (unsigned int i = 0; i < Itms.size(); i++) {
    if (Itms[i]->getName() == name) {
      Itms[i]->setActive(true, onlyParents);
    } else {
      if (Itms[i]->Items.size()) setActionActive(Itms[i]->Items, name, onlyParents);
    }
  }
}

void CAPLTreeMenu::execute() {
  if (ActionHandler != NULL) {
    if (ActionHandler->getData("APLtm").getKey() == getName()) {

      // Get the Action Vars related to the treemenu
      apl_action_data tmpHash = ActionHandler->data["APLtm"].data[getName()];
      item = tmpHash.data["Act"].value;
      oldItem = tmpHash.data["Old"].value;

      if (item.length()) {
        if (oldItem.length() || (item != getName())) executeEvent(ACTION_ACTIVE, &item);
        if (oldItem.length()) executeEvent(ACTION_ACTIVATE, &item);
        if (oldItem.length() && oldItem != item) executeEvent(ACTION_CHANGE, &item);

        ActionHandler->setVar("APLtm[" + getName() + "][Act]", item);

        setInactive(Items);
        setActionActive(Items, item, false);
      }
    }
  }
}

string CAPLTreeMenu::output() {
  APL_TRACEIN("CAPLTreeMenu::output");

  string output = "";
  if (getState() & STATE_VISIBLE) {
    setIntern("Caption", sTitle);
    prepareSlots();
    setChildren(outputItems(0, ActionHandler, this, sTarget));
    output += outputTemplate();
  }

  APL_TRACEOUT("CAPLTreeMenu::output");

  return output;
}

void CAPLTreeMenu::setCallback(int type, apl_callback func, apl_userdat_dtor dtor, void *data) {
  if (type < 0 || type > ACTION_NUMBER)
    throw runtime_error("Callback specifier exceeds limit");
  event[type].setCallback(func, dtor, data);
}

int CAPLTreeMenu::executeEvent(int type, const string *key) {
  apl_treemenu_action action;

  action.key = key;
  return event[type].execute(&action);
}
