/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: php_apl.cpp,v 1.106 2002/09/24 15:04:22 k Exp $ */


#include "php_apl.h"
#include <stdarg.h>

extern "C" {
# include "php.h"
# if defined(HAVE_ZEND_MODULES_H)
# include "Zend/modules.h"
# elif defined(HAVE_ZEND_ZEND_MODULES_H)
# include "Zend/zend_modules.h"
# endif 
}

using namespace std;

static void apl_internalAssertClass(const zval *this_ptr, const string &param, int argc, va_list ap);

vector < string > vStringVector;
vector < string > &convertToStringVector(HashTable *ht) {
  ulong iIndex;
  char *sIndex = NULL;
  zval **Data;

  vStringVector.erase(vStringVector.begin(), vStringVector.end());
  while (zend_hash_get_current_key_old(ht, &sIndex, &iIndex) != HASH_KEY_NON_EXISTANT) {
    if (sIndex) {
      efree(sIndex);
      sIndex = NULL;
    }
    zend_hash_get_current_data(ht, (void **) &Data);
    convert_to_string(*Data);
    vStringVector.push_back(Z_STRVAL_PP(Data));
    zend_hash_move_forward(ht);
  }
  return vStringVector;
}

vector < bool > vBoolVector;
vector < bool > &convertToBoolVector(HashTable *ht) {
  ulong iIndex;
  char *sIndex = NULL;
  zval **Data;

  vBoolVector.erase(vBoolVector.begin(), vBoolVector.end());
  while (zend_hash_get_current_key_old(ht, &sIndex, &iIndex) != HASH_KEY_NON_EXISTANT) {
    if (sIndex) {
      efree(sIndex);
      sIndex = NULL;
    }
    zend_hash_get_current_data(ht, (void **) &Data);
    convert_to_boolean(*Data);
    vBoolVector.push_back(zval_is_true(*Data) == 1);
    zend_hash_move_forward(ht);
  }
  return vBoolVector;
}

vector < long > vLongVector;
vector < long > &convertToLongVector(HashTable *ht) {
  ulong iIndex;
  char *sIndex = NULL;
  zval **Data;

  vLongVector.erase(vLongVector.begin(), vLongVector.end());
  while (zend_hash_get_current_key_old(ht, &sIndex, &iIndex) != HASH_KEY_NON_EXISTANT) {
    if (sIndex) {
      efree(sIndex);
      sIndex = NULL;
    }
    zend_hash_get_current_data(ht, (void **) &Data);
    convert_to_long(*Data);
    vLongVector.push_back(Z_LVAL_PP(Data));
    zend_hash_move_forward(ht);
  }
  return vLongVector;
}

vector < double > vDoubleVector;
vector < double > &convertToDoubleVector(HashTable *ht) {
  ulong iIndex;
  char *sIndex = NULL;
  zval **Data;

  vDoubleVector.erase(vDoubleVector.begin(), vDoubleVector.end());
  while (zend_hash_get_current_key_old(ht, &sIndex, &iIndex) != HASH_KEY_NON_EXISTANT) {
    if (sIndex) {
      efree(sIndex);
      sIndex = NULL;
    }
    zend_hash_get_current_data(ht, (void **) &Data);
    convert_to_double(*Data);
    vDoubleVector.push_back(Z_DVAL_PP(Data));
    zend_hash_move_forward(ht);
  }
  return vDoubleVector;
}

void aplAssertClass(const zval *this_ptr, const string &param, int argc, ...) {
  va_list ap;

  if (!APLG(lClassChecks)) return ;
  va_start(ap, argc);
  apl_internalAssertClass(this_ptr, param, argc, ap);
  va_end(ap);
}

void aplCallMethod(zval **object, const char *fname, int argc, ...) {
  zval *function;
  zval *retval;

  MAKE_STD_ZVAL(retval);
  MAKE_STD_ZVAL(function);
  ZVAL_STRING(function, (char *) fname, 1);

  va_list ap;
  va_start(ap, argc);
  if (argc == 1) {
    zval *opt;

    opt = va_arg(ap, zval *);
    apl_call_user_function(NULL, object, function, retval, argc, &opt);
  } else {
    zval **opt;

    opt = (zval **) malloc(sizeof(zval *) * argc);

    for (int i = 0; i < argc; i++) {
      opt[i] = va_arg(ap, zval *);
    }

    apl_call_user_function(NULL, object, function, retval, argc, opt);

    free(opt);
  }
  va_end(ap);

  zval_ptr_dtor(&function);
  zval_ptr_dtor(&retval);
}

void *aplGetClass(const zval *this_ptr, const char *identifier) {
  return aplGetClassEx(this_ptr, identifier, "", 0);
}

void *aplGetClassEx(const zval *this_ptr, const char *identifier, const string &param, int argc, ...) {
  void ***Class;

  if (argc) {
    va_list ap;

    va_start(ap, argc);
    apl_internalAssertClass(this_ptr, param, argc, ap);
    va_end(ap);
  }

  if (zend_hash_find(Z_OBJPROP_P(this_ptr), (char *) identifier, strlen(identifier) + 1, (void **) &Class) == FAILURE)
    zend_error(E_ERROR, "Can't find object pointer");

  return **Class;
}

zend_class_entry *aplGetClassEntry(char *name) {
  zend_class_entry *ce;

  if (zend_hash_find(EG(class_table), name, strlen(name) + 1, (void **) &ce) == FAILURE)
    zend_error(E_ERROR, "Class entry %s not found in class table", name);

  return ce;
}

bool aplIsClass(const zval *this_ptr, int argc, ...) {
  zend_class_entry *ce;
  int i;
  const char *name;
  va_list ap;
  bool retval = false;

  va_start(ap, argc);
  for (i = 0; i < argc; i++) {
    name = va_arg(ap, const char *);
    for (ce = Z_OBJCE_P(this_ptr); ce; ce = ce->parent) {
      if (!strcasecmp(ce->name, name)) {
        retval = true;
        // C++ has no other construct to break out of nested
        // loops. That is the reason for using goto here.
        // Without using goto, we would need to duplicate the
        // cleanup code here which would be even worse.
        goto cleanup;
      }
    }
  }
cleanup:
  va_end(ap);
  return retval;
}

static void apl_internalAssertClass(const zval *this_ptr, const string &param, int argc, va_list ap) {
  if (!APLG(lClassChecks)) return ;

  if (Z_TYPE_P(this_ptr) != IS_OBJECT) {
    zend_error(E_ERROR, "The %s parameter is not an object", param.c_str());
  }

  if (argc == 0) {
    zend_error(E_WARNING, "aplAssertClass was called with no allowed classes");
    return ;
  }

  int i;
  const char *name;
  string classList;
  zend_class_entry *ce;

  for (i = 0; i < argc; i++) {
    name = va_arg(ap, const char *);
    for (ce = Z_OBJCE_P(this_ptr); ce; ce = ce->parent) {
      if (!strcasecmp(ce->name, name))
        return ;
    }
    classList += name;
    if (i + 1 < argc)
      classList += ", ";
  }

  if (i == argc) {
    string errorMsg;
    string className = Z_OBJCE_P(this_ptr)->name;

    errorMsg = "The " + param + " parameter is an object of class " +
               className + ", although it should be an object of one of the "
               "following classes: " + classList;
    zend_error(E_ERROR, "%s", errorMsg.c_str());
  }
}
