/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: php_apl_actionhandler.cpp,v 1.32 2002/05/07 13:52:46 ach Exp $ */


#include "aplactionhandler.h"
#include "php_apl.h"

extern "C" {
# include "main/php.h"
};

using namespace strutilsxx;
using namespace std;

static void aplTransformActionDataExpand(zval **ActionVars,
    apl_action_hash &hash, apl_action_hash &upper_hash,
    string upperKey, int flag, int depth) {
  int iStatus;
  char *currentKey;
  ulong iKey;
  zval **data;

  zend_hash_internal_pointer_reset(Z_ARRVAL_PP(ActionVars));
  while ((iStatus = zend_hash_get_current_key_old(Z_ARRVAL_PP(ActionVars), &currentKey, &iKey)) != HASH_KEY_NON_EXISTANT) {
    if (iStatus == HASH_KEY_IS_STRING || iStatus == HASH_KEY_IS_LONG) {
      zend_hash_get_current_data(Z_ARRVAL_PP(ActionVars), (void **) & data);
      string tmpKey = "";

      switch (iStatus) {
      case HASH_KEY_IS_STRING:
        tmpKey = format(currentKey);
        break;
      case HASH_KEY_IS_LONG:
        tmpKey = intstr(iKey);
        break;
      }

      if (Z_TYPE_PP(data) == IS_ARRAY) {
#if PHP_API_VERSION < 20010119
        aplTransformActionDataExpand(data, hash, hash[upperKey].data, tmpKey, flag, depth + 1);
#else
aplTransformActionDataExpand(data, hash[tmpKey].data,
        hash[tmpKey].data, upperKey, flag, depth + 1);
#endif
      } else {
        convert_to_string_ex(data);
        upper_hash[tmpKey].data[upperKey].value = Z_STRVAL_PP(data);
      }
      if (iStatus == HASH_KEY_IS_STRING) efree(currentKey);
    }
    zend_hash_move_forward(Z_ARRVAL_PP(ActionVars));
  }
}

static void aplTransformActionData(zval **ActionVars, apl_action_hash &hash,
                                   int flag = 0, int depth = 0) {
  int iStatus;
  char *currentKey;
  ulong iKey;
  zval **data;

  zend_hash_internal_pointer_reset(Z_ARRVAL_PP(ActionVars));
  while ((iStatus = zend_hash_get_current_key_old(Z_ARRVAL_PP(ActionVars), &currentKey, &iKey)) != HASH_KEY_NON_EXISTANT) {
    if (iStatus == HASH_KEY_IS_STRING || iStatus == HASH_KEY_IS_LONG) {
      zend_hash_get_current_data(Z_ARRVAL_PP(ActionVars), (void **) & data);
      string tmpKey = "";

      switch (iStatus) {
      case HASH_KEY_IS_STRING:
        tmpKey = format(currentKey);
        break;
      case HASH_KEY_IS_LONG:
        tmpKey = intstr(iKey);
        break;
      }

      if (Z_TYPE_PP(data) == IS_ARRAY) {
        if (flag && depth == 1) {
          aplTransformActionDataExpand(data, hash, hash, tmpKey, flag, depth + 1);
        } else {
          aplTransformActionData(data, hash[tmpKey].data, flag, depth + 1);
        }
      } else {
        convert_to_string_ex(data);
        hash[tmpKey].value = Z_STRVAL_PP(data);
      }
      if (iStatus == HASH_KEY_IS_STRING) efree(currentKey);
    }
    zend_hash_move_forward(Z_ARRVAL_PP(ActionVars));
  }
}

ZEND_FUNCTION(CAPLActionHandler_CAPLActionHandler) {
  CAPLActionHandler * _result;

  switch (ZEND_NUM_ARGS()) {
  default:
    _result = (CAPLActionHandler *) new CAPLActionHandler();
  }
  zend_list_insert(_result, le_CAPLActionHandler);
  add_property_resource(this_ptr, "this", (long) _result);

  zval **ActionVars;
  CAPLActionHandler * Class;

  CATCH_ERROR_BEGIN()
  Class = (CAPLActionHandler *) aplGetClass(this_ptr);

  if (zend_hash_find(&EG(symbol_table), "HTTP_SERVER_VARS", sizeof("HTTP_SERVER_VARS"), (void **) &ActionVars) != FAILURE) {
    zval ** PHP_SELF;

    if (zend_hash_find(Z_ARRVAL_PP(ActionVars), "PHP_SELF", sizeof("PHP_SELF"), (void **) &PHP_SELF) != FAILURE) {
      convert_to_string_ex(PHP_SELF);
      Class->setPage(Z_STRVAL_PP(PHP_SELF));
    }
  }

  if (zend_hash_find(&EG(symbol_table), "HTTP_GET_VARS", sizeof("HTTP_GET_VARS"), (void **) &ActionVars) != FAILURE)
    aplTransformActionData(ActionVars, Class->data);

  if (zend_hash_find(&EG(symbol_table), "HTTP_POST_VARS", sizeof("HTTP_POST_VARS"), (void **) &ActionVars) != FAILURE)
    aplTransformActionData(ActionVars, Class->data);

  if (zend_hash_find(&EG(symbol_table), "HTTP_POST_FILES", sizeof("HTTP_POST_FILES"), (void **) &ActionVars) != FAILURE)
    aplTransformActionData(ActionVars, Class->data, 1);

  CATCH_ERROR_END()
}


static void aplCreateZvalFromHash(apl_action_hash *hash, string *value, zval *array) {
  zval *property;

  if (!hash->size()) {
    if (value) {
      ZVAL_STRINGL(array, (char *) value->c_str(), value->length(), 1);
    } else {
      ZVAL_NULL(array);
    }
    return ;
  }

  array_init(array);

  for (apl_action_hash::iterator i = hash->begin(); i != hash->end(); i++) {
    MAKE_STD_ZVAL(property);
    aplCreateZvalFromHash(&i->second.data, &i->second.value, property);
    zend_hash_update(Z_ARRVAL_P(array), (char *) i->first.c_str(),
                     i->first.length() + 1, (void *) &property, sizeof(zval *), NULL);
  }
}

ZEND_FUNCTION(CAPLActionHandler_getVar) {
  CAPLActionHandler *Class;
  zval **arg[1];

  if (ZEND_NUM_ARGS() < 0 || ZEND_NUM_ARGS() > 1
      || zend_get_parameters_array_ex(ZEND_NUM_ARGS(), arg) == FAILURE)
    WRONG_PARAM_COUNT;

  if (ZEND_NUM_ARGS()) convert_to_string_ex(arg[0]);

  CATCH_ERROR_BEGIN()
  Class = (CAPLActionHandler *) aplGetClass(this_ptr);
  if (ZEND_NUM_ARGS() > 0) {
   if (Class->existsKey(Z_STRVAL_PP(arg[0])))
     aplCreateZvalFromHash(&Class->data[Z_STRVAL_PP(arg[0])].data, &Class->data[Z_STRVAL_PP(arg[0])].value, return_value);
   else
     ZVAL_NULL(return_value);
  } else {
    aplCreateZvalFromHash(&Class->data, NULL, return_value);
  }
  CATCH_ERROR_END()
}
