/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: php_apl_dataset.cpp,v 1.30 2001/11/21 13:18:16 ach Exp $ */


#include "apldataset.h"
#include "php_apl.h"

using namespace std;

ZEND_FUNCTION(CAPLDataSet_createFields) {
  zval **arg[1];

  if (zend_get_parameters_ex(1, &arg[0])) WRONG_PARAM_COUNT;
  convert_to_array_ex(arg[0]);
  CATCH_ERROR_BEGIN()
  int iStatus;
  ulong iIndex;
  char *sIndex = NULL;
  zval **Data;
  zend_class_entry *ce;
  zval *array;

  MAKE_STD_ZVAL(array);
  array_init(array);

  ce = aplGetClassEntry("capldatasetfield");
  while ((iStatus = zend_hash_get_current_key_old(Z_ARRVAL_PP(arg[0]), &sIndex, &iIndex)) != HASH_KEY_NON_EXISTANT) {
    zval * obj;

    if (sIndex) {
      efree(sIndex);
      sIndex = NULL;
    }
    MAKE_STD_ZVAL(obj);
    object_init_ex(obj, ce);
    zend_hash_get_current_data(Z_ARRVAL_PP(arg[0]), (void **) &Data);
    convert_to_string_ex(Data);
    zend_hash_update(Z_ARRVAL_P(array), Z_STRVAL_PP(Data), Z_STRLEN_PP(Data) + 1, (void *)&obj, sizeof(zval *), NULL);
    zend_hash_move_forward(Z_ARRVAL_PP(arg[0]));
    aplCallMethod(&obj, "capldatasetfield", 1, this_ptr);
    aplCallMethod(&obj, "setname", 1, *Data);
  }
  zend_hash_update(Z_OBJPROP_P(this_ptr), "Field", sizeof("Field"), (void *) &array, sizeof(zval *), NULL);
  CATCH_ERROR_END()
}

ZEND_FUNCTION(CAPLDataSet_getList) {
  CAPLDataSet *Class;
  keylist KeyList;

  CATCH_ERROR_BEGIN()
  Class = (CAPLDataSet *) aplGetClass(this_ptr);
  KeyList = Class->getList();
  CATCH_ERROR_END()

  array_init(return_value);
  for (unsigned int i = 0; i < KeyList.size(); i++) {
    add_next_index_string(return_value, (char *) KeyList[i].c_str(), 1);
  }
}

ZEND_FUNCTION(CAPLDataSet_getData) {
  CAPLDataSet *Class;
  dataset DataSet;
  zval **arg[1];
  string _arg0 = "";
  int arg_count = ZEND_NUM_ARGS();

  if (arg_count > 1 || arg_count < 0 || zend_get_parameters_array_ex(arg_count, arg))
    WRONG_PARAM_COUNT;

  if (arg_count) {
    convert_to_string_ex(arg[0]);
    _arg0 = Z_STRVAL_PP(arg[0]);
  }

  CATCH_ERROR_BEGIN()
  Class = (CAPLDataSet *) aplGetClass(this_ptr);
  DataSet = Class->getData(_arg0);
  CATCH_ERROR_END()

  array_init(return_value);

  for (dataset::iterator Iterate = DataSet.begin(); Iterate != DataSet.end(); Iterate++) {
    add_assoc_string(return_value, (char *) (*Iterate).first.c_str(), (char *) (*Iterate).second.c_str(), 1);
  }
}

ZEND_FUNCTION(CAPLDataSet_modify) {
  CAPLDataSet *Class;
  zval **arg[2];
  map < string, string > _arg1;
  string _arg0;
  string result;

  if (zend_get_parameters_ex(2, &arg[0], &arg[1])) WRONG_PARAM_COUNT;
  convert_to_array_ex(arg[1]);
  convert_to_string_ex(arg[0]);

  _arg0 = Z_STRVAL_PP(arg[0]);
  int iStatus;
  ulong iIndex;
  char *sIndex = NULL;
  zval **Data;

  while ((iStatus = zend_hash_get_current_key_old(Z_ARRVAL_PP(arg[1]), &sIndex, &iIndex)) != HASH_KEY_NON_EXISTANT) {
    zend_hash_get_current_data(Z_ARRVAL_PP(arg[1]), (void **) & Data);
    convert_to_string_ex(Data);
    _arg1[sIndex] = Z_STRVAL_PP(Data);
    efree(sIndex);
    zend_hash_move_forward(Z_ARRVAL_PP(arg[1]));
  }

  CATCH_ERROR_BEGIN()
  Class = (CAPLDataSet *) aplGetClass(this_ptr);
  result = Class->modify(_arg0, _arg1);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) result.c_str(), result.length(), 1);
}

ZEND_FUNCTION(CAPLDataSet_add) {
  CAPLDataSet *Class;
  zval **arg[2];
  map < string, string > _arg1;
  string _arg0;
  string result;

  if (zend_get_parameters_ex(2, &arg[0], &arg[1])) WRONG_PARAM_COUNT;
  convert_to_array_ex(arg[1]);
  convert_to_string_ex(arg[0]);

  _arg0 = Z_STRVAL_PP(arg[0]);
  int iStatus;
  ulong iIndex;
  char *sIndex = NULL;
  zval **Data;

  while ((iStatus = zend_hash_get_current_key_old(Z_ARRVAL_PP(arg[1]), &sIndex, &iIndex)) != HASH_KEY_NON_EXISTANT) {
    zend_hash_get_current_data(Z_ARRVAL_PP(arg[1]), (void **) & Data);
    convert_to_string_ex(Data);
    _arg1[sIndex] = Z_STRVAL_PP(Data);
    efree(sIndex);
    zend_hash_move_forward(Z_ARRVAL_PP(arg[1]));
  }

  CATCH_ERROR_BEGIN()
  Class = (CAPLDataSet *) aplGetClass(this_ptr);
  result = Class->add(_arg0, _arg1);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) result.c_str(), result.length(), 1);
}
