/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: php_apl_reg.cpp,v 1.57 2002/03/29 22:51:26 ach Exp $ */

#include "aplexecutor.h"
#include "aplobject.h"
#include "php_apl.h"
#include "apl_config.h"
#include "apl_ac.h"
#include "apllistobject.h"

apl_global_le_list_t apl_global_le_list;

extern "C" {
# include "main/php.h"
# include "main/php_ini.h"
# include "ext/standard/info.h"
}

using namespace std;

// INI Settings

static PHP_INI_MH(OnUpdateCPPString) {
  char **p;
  char *base = (char *) mh_arg2;
  string *str;

  p = (char **) (base + (size_t) mh_arg1);
  str = (string *) p;
  *str = new_value;

  return SUCCESS;
}

PHP_INI_BEGIN()
STD_PHP_INI_ENTRY("apl.template_dir", APL_TEMPLATE_DIR, PHP_INI_ALL, OnUpdateCPPString, sTemplateRoot, zend_apl_globals, apl_globals)
STD_PHP_INI_ENTRY("apl.template_alias", APL_TEMPLATE_ALIAS, PHP_INI_ALL, OnUpdateCPPString, sTemplateAlias, zend_apl_globals, apl_globals)
STD_PHP_INI_ENTRY("apl.template_type", TEMPLATETYPE, PHP_INI_ALL, OnUpdateCPPString, sTemplateType, zend_apl_globals, apl_globals)
STD_PHP_INI_ENTRY("apl.theme", THEME, PHP_INI_ALL, OnUpdateCPPString, sTheme, zend_apl_globals, apl_globals)
STD_PHP_INI_ENTRY("apl.style", STYLE, PHP_INI_ALL, OnUpdateCPPString, sStyle, zend_apl_globals, apl_globals)
STD_PHP_INI_ENTRY("apl.style_file", STYLE_CONF, PHP_INI_ALL, OnUpdateCPPString, styleFile, zend_apl_globals, apl_globals)
STD_PHP_INI_ENTRY("apl.class_checks", CLASS_CHECKS, PHP_INI_ALL, OnUpdateInt, lClassChecks, zend_apl_globals, apl_globals)
PHP_INI_END()

// Creation of method-arrays

#include "php_if_decl.h"
#include "php_if_table.h"

// Creation of function-array

zend_function_entry functions[] = {
  { "aplbold", ZEND_FN(APLBold), NULL},
  { "aplunderline", ZEND_FN(APLUnderline), NULL},
  { "aplitalic", ZEND_FN(APLItalic), NULL},
  { "aplstrike", ZEND_FN(APLStrike), NULL},
  { "aplheadline", ZEND_FN(APLHeadline), NULL},
  { "aplhyperlink", ZEND_FN(APLHyperLink), NULL},
  { "aplbreak", ZEND_FN(APLBreak), NULL},
  { "aplnobreak", ZEND_FN(APLNoBreak), NULL},
  { "aplwordbreak", ZEND_FN(APLWordBreak), NULL},
  {NULL, NULL, NULL }
};

// Handle & destructor-call for each class

#include "php_if_dtors.h"

// Init functions

#include "apldatacontrolobject.h"

#define REG_GLOBAL_ID(a) REGISTER_LONG_CONSTANT("APL_" #a, a, CONST_PERSISTENT)

ZEND_MINIT_FUNCTION(apl) {
  zend_class_entry e;

# include "php_if_minit.h"

  REG_GLOBAL_ID(EVENT_ADD_BEFORE);
  REG_GLOBAL_ID(EVENT_ADD_AFTER);
  REG_GLOBAL_ID(EVENT_MODIFY_BEFORE);
  REG_GLOBAL_ID(EVENT_MODIFY_AFTER);
  REG_GLOBAL_ID(EVENT_REMOVE_BEFORE);
  REG_GLOBAL_ID(EVENT_REMOVE_AFTER);
  REG_GLOBAL_ID(ACTION_ACTIVATE);
  REG_GLOBAL_ID(ACTION_ACTIVE);
  REG_GLOBAL_ID(ACTION_CHANGE);
  REG_GLOBAL_ID(ACTION_MISC);
  REG_GLOBAL_ID(ACTION_ADD);
  REG_GLOBAL_ID(ACTION_ADD_ASK);
  REG_GLOBAL_ID(ACTION_MODIFY);
  REG_GLOBAL_ID(ACTION_MODIFY_ASK);
  REG_GLOBAL_ID(ACTION_REMOVE);
  REG_GLOBAL_ID(ACTION_REMOVE_ASK);
  REG_GLOBAL_ID(ACTION_SAVE);
  REG_GLOBAL_ID(ACTION_SAVE_NEW);
  REG_GLOBAL_ID(ACTION_CANCEL);
  REG_GLOBAL_ID(ACTION_STANDARD);
  REG_GLOBAL_ID(ACTION_FILTER);
  REG_GLOBAL_ID(ACTION_ORDER);
  REG_GLOBAL_ID(ACTION_SKIP);
  REG_GLOBAL_ID(CONVERTOR_OUTPUT);
  REG_GLOBAL_ID(CONVERTOR_STORAGE);
  REG_GLOBAL_ID(CONVERTOR_ACTION);
  REG_GLOBAL_ID(STATE_VISIBLE);
  REG_GLOBAL_ID(STATE_HIDDEN);
  REG_GLOBAL_ID(STATE_ENABLED);
  REG_GLOBAL_ID(STATE_DISABLED);
  REG_GLOBAL_ID(STATE_READWRITE);
  REG_GLOBAL_ID(STATE_READONLY);
  REG_GLOBAL_ID(DATACONTROL_FETCH);
  REG_GLOBAL_ID(DATACONTROL_UPDATE);
  REG_GLOBAL_ID(DATACONTROL_ADD);
  REG_GLOBAL_ID(DATACONTROL_MODIFY);
  REG_GLOBAL_ID(DATACONTROL_REMOVE);
  REG_GLOBAL_ID(VALIDATOR_STANDARD);
  REG_GLOBAL_ID(VALIDATOR_COLLECTOR);
  REG_GLOBAL_ID(FILTER_STANDARD);
  REG_GLOBAL_ID(FILTER_WILDCARD);
  REG_GLOBAL_ID(DATABASE_ODBC);
  REG_GLOBAL_ID(DATABASE_MYSQL);
  REG_GLOBAL_ID(DATABASE_POSTGRES);
  REG_GLOBAL_ID(DATABASE_STANDARD);
  REG_GLOBAL_ID(DATETIME_STANDARD);
  REG_GLOBAL_ID(DATETIME_DATE);
  REG_GLOBAL_ID(DATETIME_TIME);
  REG_GLOBAL_ID(EDITBOX_STANDARD);
  REG_GLOBAL_ID(EDITBOX_PASSWORD);
  REG_GLOBAL_ID(BUTTON_STANDARD);
  REG_GLOBAL_ID(BUTTON_IMAGE);
  REG_GLOBAL_ID(BUTTON_RESET);
  REG_GLOBAL_ID(SHAPE_POLYGON);
  REG_GLOBAL_ID(SHAPE_RECT);
  REG_GLOBAL_ID(SHAPE_CIRCLE);
  REG_GLOBAL_ID(CHECKBOX_STANDARD);
  REG_GLOBAL_ID(CHECKBOX_BOOLEAN);

  REGISTER_INI_ENTRIES();

  return SUCCESS;
}

ZEND_MSHUTDOWN_FUNCTION(apl) {
  UNREGISTER_INI_ENTRIES();
  return SUCCESS;
}

APLG_DECLARE;

ZEND_MINFO_FUNCTION(apl) {

  php_info_print_table_start();
  php_info_print_table_row(2, "APL Version", APL_VERSION);
  php_info_print_table_row(2, "APL Release", APL_RELEASE);
  php_info_print_table_end();

  DISPLAY_INI_ENTRIES();
}

ZEND_RINIT_FUNCTION(apl) {
  CAPLObject::initUniqueName();
  return SUCCESS;
}

ZEND_RSHUTDOWN_FUNCTION(apl) {
  CAPLExecutor::clearGlobal();
  return SUCCESS;
}

zend_module_entry apl_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
  STANDARD_MODULE_HEADER,
#endif
  "apl",
  functions,
  ZEND_MINIT(apl),
  ZEND_MSHUTDOWN(apl),
  ZEND_RINIT(apl),
  ZEND_RSHUTDOWN(apl),
  ZEND_MINFO(apl),
#if ZEND_MODULE_API_NO >= 20010901
  APL_VERSION,
#endif
  STANDARD_MODULE_PROPERTIES
};

extern "C" {
  ZEND_GET_MODULE(apl)
}
