/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: php_apl_text_funcs.cpp,v 1.10 2001/11/21 13:18:16 ach Exp $ */


#include "php_apl.h"
#include "apltext.h"

using namespace std;

ZEND_FUNCTION(APLBold) {
  zval **arg[1];
  string _arg0;
  string sReturn;

  if (zend_get_parameters_ex(1, &arg[0])) WRONG_PARAM_COUNT;
  convert_to_string_ex(arg[0]);
  _arg0 = Z_STRVAL_PP(arg[0]);
  CATCH_ERROR_BEGIN()
  sReturn = APLBold(_arg0);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}

ZEND_FUNCTION(APLUnderline) {
  zval **arg[1];
  string _arg0;
  string sReturn;

  if (zend_get_parameters_ex(1, &arg[0])) WRONG_PARAM_COUNT;
  convert_to_string_ex(arg[0]);
  _arg0 = Z_STRVAL_PP(arg[0]);
  CATCH_ERROR_BEGIN()
  sReturn = APLUnderline(_arg0);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}

ZEND_FUNCTION(APLItalic) {
  zval **arg[1];
  string _arg0;
  string sReturn;

  if (zend_get_parameters_ex(1, &arg[0])) WRONG_PARAM_COUNT;
  convert_to_string_ex(arg[0]);
  _arg0 = Z_STRVAL_PP(arg[0]);
  CATCH_ERROR_BEGIN()
  sReturn = APLItalic(_arg0);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}

ZEND_FUNCTION(APLStrike) {
  zval **arg[1];
  string _arg0;
  string sReturn;

  if (zend_get_parameters_ex(1, &arg[0])) WRONG_PARAM_COUNT;
  convert_to_string_ex(arg[0]);
  _arg0 = Z_STRVAL_PP(arg[0]);
  CATCH_ERROR_BEGIN()
  sReturn = APLStrike(_arg0);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);

}

ZEND_FUNCTION(APLHeadline) {
  zval **args[2];
  string _arg0;
  int _arg1 = 3;
  string sReturn;
  int arg_count = ZEND_NUM_ARGS();

  if (arg_count > 2 || arg_count < 1 || zend_get_parameters_array_ex(arg_count, args))
    WRONG_PARAM_COUNT;

  switch (arg_count) {
  case 2:
    convert_to_long_ex(args[1]);
    _arg1 = (*args[1])->value.lval;
  case 1:
    convert_to_string_ex(args[0]);
    _arg0 = Z_STRVAL_PP(args[0]);
  }
  CATCH_ERROR_BEGIN()
  sReturn = APLHeadline(_arg0, _arg1);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}

ZEND_FUNCTION(APLHyperLink) {
  zval **args[3];
  string sLink;
  string sText = "";
  string sTarget = "";
  string sReturn;
  int arg_count = ZEND_NUM_ARGS();

  if (arg_count > 3 || arg_count < 1 || zend_get_parameters_array_ex(arg_count, args))
    WRONG_PARAM_COUNT;

  switch (arg_count) {
  case 3:
    convert_to_string_ex(args[2]);
    sTarget = Z_STRVAL_PP(args[2]);
  case 2:
    convert_to_string_ex(args[1]);
    sText = Z_STRVAL_PP(args[1]);
  case 1:
    convert_to_string_ex(args[0]);
    sLink = Z_STRVAL_PP(args[0]);
  }
  CATCH_ERROR_BEGIN()
  sReturn = APLHyperLink(sLink, sText, sTarget);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}

ZEND_FUNCTION(APLNoBreak) {
  zval **arg[1];
  string _arg0;
  string sReturn;

  if (zend_get_parameters_ex(1, &arg[0])) WRONG_PARAM_COUNT;
  convert_to_string_ex(arg[0]);
  _arg0 = Z_STRVAL_PP(arg[0]);
  CATCH_ERROR_BEGIN()
  sReturn = APLNoBreak(_arg0);
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}

ZEND_FUNCTION(APLBreak) {
  string sReturn;
  zval **arg[1];

  if (ZEND_NUM_ARGS() < 0 || ZEND_NUM_ARGS() > 1
      || zend_get_parameters_array_ex(ZEND_NUM_ARGS(), arg) == FAILURE) WRONG_PARAM_COUNT;
  if (ZEND_NUM_ARGS() > 0) convert_to_string_ex(arg[0]);

  CATCH_ERROR_BEGIN()
  sReturn = APLBreak(ZEND_NUM_ARGS() > 0 ? Z_STRVAL_PP(arg[0]) : "");
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}

ZEND_FUNCTION(APLWordBreak) {
  string sReturn;

  CATCH_ERROR_BEGIN()
  sReturn = APLWordBreak();
  CATCH_ERROR_END()
  RETVAL_STRINGL((char *) sReturn.c_str(), sReturn.length(), 1);
}
