/*************************************************************************
* apl - abstract presentation layer                                      *
* ---------------------------------------------------------------------- *
* Copyright (C) 2000 - 2001 twisd AG, <http://www.twisd.de>              *
* See AUTHORS file for a list of contributing developers                 *
*                                                                        *
* This library is free software; you can redistribute it and/or          *
* modify it under the terms of the GNU Lesser General Public             *
* License as published by the Free Software Foundation; either           *
* version 2 of the License, or (at your option) any later version.       *
*                                                                        *
* This library is distributed in the hope that it will be useful,        *
* but WITHOUT ANY WARRANTY; without even the implied warranty of         *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
* Library General Public License for more details.                       *
*                                                                        *
* You should have received a copy of the GNU Lesser General Public       *
* License along with this progam; if not, write to the                   *
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,           *
* Boston, MA 02111-1307, USA.                                            *
*************************************************************************/

/* $Id: php_apl_treemenu.cpp,v 1.9 2001/11/21 13:18:16 ach Exp $ */


#include "apltreemenu.h"
#include "php_apl.h"
#include "php_apl_calls.h"

extern "C" {
# include "php.h"
}

using namespace std;

ZEND_FUNCTION(CAPLTreeMenu_addItem) {
  zval **args[4];
  int arg_count = ZEND_NUM_ARGS();

  if (arg_count > 4 || arg_count < 2 || zend_get_parameters_array_ex(arg_count, args))
    WRONG_PARAM_COUNT;

  switch (arg_count) {
  case 4:
    convert_to_string_ex(args[3]);
  case 3:
    convert_to_string_ex(args[2]);
    break;
  }
  convert_to_string_ex(args[1]);
  convert_to_string_ex(args[0]);

  CATCH_ERROR_BEGIN()
  zend_class_entry *ce;
  zval *array;
  zval *obj;
  zval **arrayp;

  if (zend_hash_find(Z_OBJPROP_P(this_ptr), "Item", sizeof("Item"), (void **) &arrayp) != SUCCESS) {
    MAKE_STD_ZVAL(array);
    array_init(array);
    zend_hash_update(Z_OBJPROP_P(this_ptr), "Item", sizeof("Item"), (void *) &array, sizeof(zval *), NULL);
  } else {
    array = *arrayp;
  }

  ce = aplGetClassEntry("capltreemenuitem");
  MAKE_STD_ZVAL(obj);
  object_init_ex(obj, ce);
  zend_hash_update(Z_ARRVAL_P(array), Z_STRVAL_PP(args[0]), Z_STRLEN_PP(args[0]) + 1, (void *)&obj, sizeof(zval *), NULL);
  aplCallMethod(&obj, "capltreemenuitem", 1, this_ptr);
  aplCallMethod(&obj, "setname", 1, *args[0]);
  aplCallMethod(&obj, "settext", 1, *args[1]);
  switch (arg_count) {
  case 4:
    aplCallMethod(&obj, "setimage", 1, *args[3]);
  case 3:
    aplCallMethod(&obj, "setlink", 1, *args[2]);
  }
  CATCH_ERROR_END()
}

static int aplTreeMenuCallback(void *dummy, void *dummy2) {
  zval *args[1];
  apl_php_callback_t *php_dat = (apl_php_callback_t *) dummy;
  apl_treemenu_action *apl_dat = (apl_treemenu_action *) dummy2;
  int ret;

  aplCreateArguments(1, args);
  ZVAL_STRINGL(args[0], (char *) apl_dat->key->c_str(), apl_dat->key->length(), 1);
  ret = aplCallHandler(php_dat->func, 1, args);
  aplDestroyArguments(1, args);

  return ret;
}

ZEND_FUNCTION(CAPLTreeMenu_setCallback) {
  CAPLTreeMenu *Class;
  zval **arg[2];
  apl_php_callback_t *dat;

  if (zend_get_parameters_array_ex(2, arg)) WRONG_PARAM_COUNT;
  convert_to_long_ex(arg[0]);
  if (Z_LVAL_PP(arg[0]) < 0 || Z_LVAL_PP(arg[0]) >= ACTION_NUMBER) {
    php_error(E_WARNING, "Action number exceeds limit");
    return ;
  }
  ZVAL_ADDREF(*arg[1]);
  dat = (apl_php_callback_t *) emalloc(sizeof(*dat));
  dat->func = *arg[1];
  CATCH_ERROR_BEGIN()
  Class = (CAPLTreeMenu *) aplGetClass(this_ptr);
  Class->setCallback(Z_LVAL_PP(arg[0]), aplTreeMenuCallback, aplCallbackDestructor, dat);
  CATCH_ERROR_END()
}
